
      PROGRAM NICOCAL
C----------------------------------------------------------------------
C Name & Purpose:
C ---------------
C Nonlinear Impulse-response Carbon-cycle Ocean-box-model CALibration
C (NICOCAL)
C 1. Read all CarC & CliC IRF parameters
C    & ocean chemistry parameters
C
C 2. compute SigmaC_0 & Xibuf
C    to close chemical equilibrium.
C
C 3. compute ocean CarC boxmodel parameters
C    (layer thicknesses & diffusion coefficients).
C
C 4. write complete input file to niccs/liccs/nicaos
C
C Earlier name:
C -------------
C tune.F
C
C References:
C -----------
C E.Maier-Reimer: pers.comm. (1997).
C R.Bacastow: "Numerical evaluation of the evasion factor", section 3.4
C of B.Bolin (ed): "Carbon cycle modelling" (SCOPE 16, 1981).
C
C Author:
C -------
C G.Hooss, MPI Me, Nov 1997, ..., Nov 2001.
C
C
C Copyright
C ---------
C (C) 2001 K.G.Hooss
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License
C as published by the Free Software Foundation; either version 2
C of the License, or (at your option) any later version.
C 
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C 
C You should have received a copy of the GNU General Public License
C along with this program; if not, write to the Free Software
C Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
C----------------------------------------------------------------------

      IMPLICIT NONE

#include "scmatbox.h"
#include "NICOirfs.h"

C----------------------------------------------------------------------
C Read parameters.
C----------------------------------------------------------------------

      CALL INPPAR

C----------------------------------------------------------------------
C Compute preindustrial DIC and buffer factor.
C----------------------------------------------------------------------

      CALL CHEM

C----------------------------------------------------------------------
C Test: output initial parameter values.
C----------------------------------------------------------------------
c
c      CALL MKINP
c
C----------------------------------------------------------------------
C Make combilayer IRF & thicknesses (Ha, Hs, H1)
C----------------------------------------------------------------------

      CALL MKCOMB(SIGC0, XIBUF, HH(1), HS)

C----------------------------------------------------------------------
C First-guess values to layer thicknesses [in m] (HLAYER)
C & diffusion coefficients [in m/y] (ETA).
C Note (1) that the first diffusion coefficient ETA(1) is not used by
C the model and will thus be ignored by the tuning routine,
C and (2) that the 1st layer thickness H(1) will remain unchanged which
C is allowed because the system is invariant under scaling of all
C parameters by a common factor,
C and (3) that the 2nd layer thickness H(2) cannot be choosen freely
C but depends on the choice of h1, h3, & h4; the value given here will
C be ignored by the tuning rougine.
C So the only relevant numbers are HL(1) (fixed), HL(3,4),
C and ETA(2,3,4).
C----------------------------------------------------------------------
c
c      HH(1)  =  663.8
c      HH(2)  =    0.0
c      HH(3)  = 1600.0
c      HH(4)  = 6000.0
c
c      ETA(1) =    0.0
c      ETA(2) =    5.0
c      ETA(3) =    5.0
c      ETA(4) =   15.0
c
C----------------------------------------------------------------------
C Box model parameters.
C----------------------------------------------------------------------

      CALL MKBOX(HH, ETA)

C----------------------------------------------------------------------
C Final ASCII output in the format needed as NICCS input file.
C----------------------------------------------------------------------

      CALL MKINP

C----------------------------------------------------------------------

      END

C######################################################################

      SUBROUTINE MKBOX(HL, ETA)
C----------------------------------------------------------------------
C Search box model parameters (layer thicknesses & diffusion coeffs).
C from combilayer's reduced IRF and thickness HH(1).
C
C G.Hooss, MPI Me, Nov 1997.
C----------------------------------------------------------------------

      IMPLICIT NONE

      REAL HL(4), ETA(4)

C----------------------------------------------------------------------
c      INTEGER I
C----------------------------------------------------------------------

#include "NICOirfs.h"

C----------------------------------------------------------------------
C Test output of first-guess box model parameters.
C----------------------------------------------------------------------
c      write(*,*)
c      write(*,*) 'mkbox'
c      write(*,*)
c      write(*,*) 'HL'
c      write(*,*) HL(1), HL(2), HL(3), HL(4)
c      write(*,*) 'ETA'
c      write(*,*) ETA(1), ETA(2), ETA(3), ETA(4)
c      write(*,*) 'c2'
c      write(*,*)
C----------------------------------------------------------------------
C Test output of combi IRF.
C----------------------------------------------------------------------
c      WRITE(*,*)
c      WRITE(*,*) 'RCT'
c      WRITE(*,'(3F16.8)') (RCT(I), I=1,3)
c      WRITE(*,*)
c      WRITE(*,*) 'RCA, SUM'
c      WRITE(*,'(5F16.8)') (RCA(I), I=1,4), RCA(1)+RCA(2)+RCA(3)+RCA(4)
c      WRITE(*,*)
C----------------------------------------------------------------------

      CALL ROOTS(RCA, RCT, HL, ETA)

C----------------------------------------------------------------------

      RETURN
      END

C######################################################################

      SUBROUTINE MKCOMB(SIGC0, XIBUF, H1, HS)
C----------------------------------------------------------------------
C Compute combilayer's reduced IRF
C and 3 thicknesses (total, atm, mixed-layer).
C
C G.Hooss, MPI Me, Nov 1997.
C----------------------------------------------------------------------

      REAL SIGC0, XIBUF, H1, HS

      REAL MO2GT
      PARAMETER(MO2GT = 12.001E-12)

#include "NICOirfs.h"
#include "scparche.h"

      REAL AW4SF1, HA

C----------------------------------------------------------------------
C 1st time scale is the shortest, last (4th) is infinity.
C----------------------------------------------------------------------

      RCT(1) = RWT(3)
      RCT(2) = RWT(2)
      RCT(3) = RWT(1)

C----------------------------------------------------------------------
C Scale amplitudes.
C----------------------------------------------------------------------

      AW4SF1 = 1.0 / (1.0 - RWA(4))

      RCA(1) = RWA(3) * AW4SF1
      RCA(2) = RWA(2) * AW4SF1
      RCA(3) = RWA(1) * AW4SF1
      RCA(4) = RWA(0) * AW4SF1

C----------------------------------------------------------------------
C Thickness of virtual atmospheric layer. Note that this is local.
C----------------------------------------------------------------------

      HA = W0 / (MO2GT * SIGC0 * AOC)

C----------------------------------------------------------------------
C Total thickness of combined layer.
C----------------------------------------------------------------------

      H1 = AW4SF1 * XIBUF * HA

C----------------------------------------------------------------------
C Mixed layer.
C----------------------------------------------------------------------

      HS = RWA(4) * H1

C----------------------------------------------------------------------

      RETURN
      END

C######################################################################

      SUBROUTINE MKINP
C----------------------------------------------------------------------
C Write input file to NICCS & Co models.
C All NICCS input parameters are given to this routine as arguments.
C
C G.Hooss, MPI Me, Nov 1997, Oct 1999, Nov 2001.
C----------------------------------------------------------------------

      IMPLICIT NONE

#include "scmatbox.h"
#include "scparbio.h"
#include "scparche.h"
#include "scpareco.h"
#include "scsensit.h"
#include "NICOirfs.h"

      INTEGER I
      CHARACTER*72 C1, C2
C----------------------------------------------------------------------
C Initial comments.
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C2 = 'nerated by NICOCAL.'
      C1 = '# Input to NICCS & Co models, ge' // C2
      WRITE(*,'(A70)') C1

      C2 = ' pipe into NICCS / LICCS / NICAOS.'
      C1 = '# Filter this by "grep -v ''#''",' // C2
      WRITE(*,'(A70)') C1
      C1 =  '#'
      WRITE(*,'(A70)') C1

C----------------------------------------------------------------------
C R_w
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2

      WRITE(*,'(A14)') '# Carbon cycle'

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2

      WRITE(*,'(A5)') '# R_w'

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 = '# RW ampl: RWA0'
      WRITE(*,'(A15,4A15)') C1, 'RWA1', 'RWA2', 'RWA3', 'RWA4'
      WRITE(*,'(5F15.5)') (RWA(I), I=0,4)

      C1 = '# RW time:      '
      WRITE(*,'(A15,4A15)') C1, 'RWT1', 'RWT2', 'RWT3', 'RWT4'
      WRITE(*,'(A15,4F15.5)') ' ', (RWT(I), I=1,4)

C----------------------------------------------------------------------
C Chemistry parameters.
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2


      C1 = '# Ocean chemistry'
      C2 = C1 // ' (get consistent SigmaC_0 and Xibuf using xibuf.F)'
      WRITE(*,'(A67)') C2

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 = '# Chem.:       '
      WRITE(*,'(5A15)') C1, 'PCO2_0/ppmv', 'ALK', 'Total B', 'AOC'
      WRITE(*,'(A15, 4E15.6)') ' ', ALPHAA*W0, ALK, BTS, AOC

      C1 = '# Chem.:       '
      WRITE(*,'(5A15)') C1, 'K1', 'K2', 'KB', 'KW'
      WRITE(*,'(A15, 4E15.6)') ' ', AK1, AK2, AKB, AKW

      C1 = '# Chem.:       '
      WRITE(*,'(5A15)') C1, 'AlphaA', 'AlphaS', 'SigmaC_0', 'Xibuf'
      WRITE(*,'(A15,2E15.6,2E15.8)')
     .     ' ', ALPHAA, ALPHAS, SIGC0, XIBUF

C----------------------------------------------------------------------
C Land biosphere NPP fertilization parameters.
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2


      C1 = '# Land biosphere fertilization parameters'
      WRITE(*,'(A41)') C1

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 = '# Land biosph: '
      WRITE(*,'(A15,2A15)') C1, 'FNPP0', 'BETANPP'
      WRITE(*,'(A15,2F15.8)') ' ', FNPP0, BETANPP

C----------------------------------------------------------------------
C R_decay
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2


      C1 = '# R_decay'
      WRITE(*,'(A9)') C1

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 = '# Rdecay ampl: '
      WRITE(*,'(A15,4A15)') C1, 'RDA1', 'RDA2', 'RDA3', 'RDA4'
      WRITE(*,'(A15,4F15.8)') ' ', (RDA(I), I=1,4)

      C1 = '# Rdecay time: '
      WRITE(*,'(A15,4A15)') C1, 'RDT1', 'RDT2', 'RDT3', 'RDT4'
      WRITE(*,'(A15,4F15.8)') ' ', (RDT(I), I=1,4)

C----------------------------------------------------------------------
C R_PC*
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2


      C1 = '# Climate change'
      WRITE(*,'(A16)') C1

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 =  '# T2m sensitivity:            '
      WRITE(*,'(A30)') C1
      WRITE(*,'(A15,F15.8)') ' ', SENSIT2M

      C1 =  '# RPCT ampl:   '
      WRITE(*,'(A15,2A15)') C1, 'RPCTA1', 'RPCTA2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCTA(I), I=1,2)

      C1 = '# RPCT time:    '
      WRITE(*,'(A15,2A15)') C1, 'RPCTT1', 'RPCTT2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCTT(I), I=1,2)

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 =  '# Cloud coverage sensitivity: '
      WRITE(*,'(A30)') C1
      WRITE(*,'(A15,F15.8)') ' ', SENSICLO

      C1 =  '# RPCC ampl:   '
      WRITE(*,'(A15,2A15)') C1, 'RPCCA1', 'RPCCA2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCCA(I), I=1,2)

      C1 = '# RPCC time:    '
      WRITE(*,'(A15,2A15)') C1, 'RPCCT1', 'RPCCT2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCCT(I), I=1,2)

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 =  '# Precipitation sensitivity:  '
      WRITE(*,'(A30)') C1
      WRITE(*,'(A15,F15.8)') ' ', SENSIPRE

      C1 =  '# RPCP ampl:   '
      WRITE(*,'(A15,2A15)') C1, 'RPCPA1', 'RPCPA2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCPA(I), I=1,2)

      C1 = '# RPCP time:    '
      WRITE(*,'(A15,2A15)') C1, 'RPCPT1', 'RPCPT2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCPT(I), I=1,2)

      C1 = '#'
      C2 = '-------------------------------------'
      WRITE(*,'(A1,2A37)') C1, C2, C2

      C1 =  '# Sea level sensitivity:      '
      WRITE(*,'(A30)') C1
      WRITE(*,'(A15,F15.8)') ' ', SENSISEA

      C1 =  '# RPCS ampl:   '
      WRITE(*,'(A15,2A15)') C1, 'RPCSA1', 'RPCSA2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCSA(I), I=1,2)

      C1 = '# RPCS time:    '
      WRITE(*,'(A15,2A15)') C1, 'RPCST1', 'RPCST2'
      WRITE(*,'(A15,2F15.8)') ' ', (RPCST(I), I=1,2)

C----------------------------------------------------------------------
C Box model parameters: Layer thicknesses & diffusion coefficients.
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2


      C1 = '# Tuned boxmodel parameters'
      WRITE(*,'(A27)') C1

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2

      WRITE(*,'(5A15)') '# Layers:    H1', 'H2', 'H3', 'H4', 'Hsurf'
      WRITE(*,'(5E15.8)') (HH(I), I=1,4), HS

      WRITE(*,'(4A15)') '# Diffus:  ETA1', 'ETA2', 'ETA3', 'ETA4'
      WRITE(*,'(4E15.8)') (ETA(I), I=1,4)

C----------------------------------------------------------------------
C Final comments.
C----------------------------------------------------------------------

      C1 = '#'
      C2 = '====================================='
      WRITE(*,'(A1,2A37)') C1, C2, C2

C----------------------------------------------------------------------

      RETURN
      END

C######################################################################

      SUBROUTINE INPPAR
C----------------------------------------------------------------------
C Read parameters (in NICCS-input-like format) from stdin.
C NICCS-input-like file has to be filtered by "grep -v '#'".
C
C G.Hooss, MPI Me, Nov 1997, Oct 1999, Nov 2001.
C----------------------------------------------------------------------

      IMPLICIT NONE

      REAL PCO2_0

#include "scmatbox.h"
#include "scparbio.h"
#include "scparche.h"
#include "scpareco.h"
#include "scsensit.h"
#include "NICOirfs.h"

      INTEGER I
C----------------------------------------------------------------------
C IRF parameters: Amplitudes & time constants to R_w.
C----------------------------------------------------------------------

      READ(*,*) (RWA(I), I=0,4)
      READ(*,*) (RWT(I), I=1,4)

C----------------------------------------------------------------------
C Chemistry parameters.
C----------------------------------------------------------------------

      READ(*,*) PCO2_0, ALK, BTS, AOC
      READ(*,*) AK1, AK2, AKB, AKW
      READ(*,*) ALPHAA, ALPHAS, SIGC0, XIBUF

      W0 = PCO2_0 / ALPHAA

C----------------------------------------------------------------------
C Land biosphere parameters.
C----------------------------------------------------------------------

      READ(*,*) FNPP0, BETANPP

C----------------------------------------------------------------------
C R_decay.
C----------------------------------------------------------------------

      READ(*,*) (RDA(I), I=1,4)
      READ(*,*) (RDT(I), I=1,4)

C----------------------------------------------------------------------
C R_T & co.
C----------------------------------------------------------------------

      READ(*,*) SENSIT2M
      READ(*,*) (RPCTA(I), I=1,2)
      READ(*,*) (RPCTT(I), I=1,2) 
      READ(*,*) SENSICLO
      READ(*,*) (RPCCA(I), I=1,2)
      READ(*,*) (RPCCT(I), I=1,2)
      READ(*,*) SENSIPRE
      READ(*,*) (RPCPA(I), I=1,2)
      READ(*,*) (RPCPT(I), I=1,2)
      READ(*,*) SENSISEA
      READ(*,*) (RPCSA(I), I=1,2)
      READ(*,*) (RPCST(I), I=1,2)

C======================================================================
C 1st guess: Layer thicknesses & diffusion coefficients.
C----------------------------------------------------------------------

      READ(*,*) (HH(I), I=1,4), HS
      READ(*,*) (ETA(I), I=1,4)

C----------------------------------------------------------------------

      RETURN
      END

C== end ===============================================================
