C----------------------------------------------------------------------
C subroutines to the NICOCAL program.
C######################################################################

      SUBROUTINE ROOTS(RWA, RWT, HL, ETA)
C----------------------------------------------------------------------
C Purpose:
C --------
C Find Layer thicknesses & diffusion coefficients for box-diffusion
C analogue that serves as CO2 module in the SIAM climate model.
C (Former mkmod program.)
C
C Interface:
C ----------
C     CALL ROOTS(RWA, RWT, HL, ETA)
C
C Arguments:
C	RWA	Vec, on entry: combilayer (reduced) IRF amplitudes.
C	RWT	Vec, on entry: combilayer (reduced) IRF time constants.
C	HL	Vec, layer thicknesses. On entry: 1st guess,
C			on exit: optimized.
C	ETA	Vec, diffusion coefficients. On entry: 1st guess,
C			on exit: optimized.
C
C Fixed parameters:
C	LWA	Length of REAL workspace WA for optimization routine.
C	NSCAL	Max power to which 1st guess values of thicknesses &
C		 diff coeffs are scaled (in both directions)
C		 by repeated application of SCALFAC.
C
C	XTOL	Widens the range of 1st-guess values (for rounding).
C	SCALFAC	Factor by which 1st guess values are scaled (log grid).
C	OUTTOL	Max value of cost function for acceptable solution.
C
C Local variables:
C	HLMIN	vec of layer thicknesses (min values).
C	HLMAX	vec of layer thicknesses (max values).
C	HLOPT	vec of layer thicknesses (optimized).
C	ETAMIN	Vec of diffusion coefficients (min values).
C	ETAMAX	Vec of diffusion coefficients (max values).
C	ETAOPT	Vec of diffusion coefficients (optimized).
C	XCONT	Vec of control variables.
C	FUNCTS	Vec of functions of which the zeroes are searched.
C	J,K	Counting indices.
C	IFAIL	Error indicator for NAG routines.
C	NEGFLAG	Error indicator for negative parameters.
C	MARKSOL	Flag indicating that a solution is found.
C	FSUMSQ	Cost function.
C	EPCHECK	Vec of emission pulse backtransformed into box coord.
C	WA	Workspace array for root finding routine.
C
C
C Space of independent parameters is reduced to 5 dimensions
C (2 layer thicknesses and 3 diffusion coefficients) by the following
C constraints:
C
C (1) hlayer(1) = ha + hs = const
C    ha is a constant of the system, hs must be chosen to a certain
C    fixed value to fulfill the equilibrium condition for the partial
C    pressures in atmosphere & surface water. This is allowed in the
C    tuning procedure by the scaling invariance of the system matrix.
C
C (2) sum(hlayers) = hlayer(1) / a(0)
C    because equil. C distribution is given by layer thickness ratios. 
C
C Method:
C -------
C Seek simultaneous roots of 2N-3 nonlinear equations in 2N-3 model
C parameters. The functions are computed by a subroutine FUNC1 that
C can be called by the NAG routine C05NBF.
C
C Try from a grid of starting points in the 5-dim space of
C eta2 - eta4, hl3, hl4.
C
C Externals:
C ----------
C	C05NBF	Seek simultaneous roots of nonlinear functions.
C	MKMOUT	For checking purposes: output results to stdout.
C	FUNC1	Evaluate nonlinear functions.
C
C Author:
C -------
C G.Hooss, MPI Me, Nov 1997
C
C
C Copyright
C ---------
C (C) 2001 K.G.Hooss
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License
C as published by the Free Software Foundation; either version 2
C of the License, or (at your option) any later version.
C 
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C 
C You should have received a copy of the GNU General Public License
C along with this program; if not, write to the Free Software
C Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
C----------------------------------------------------------------------

      IMPLICIT NONE

#include "NICOroot.h"

      REAL*8 RWA(N), RWT(N), HL(N), ETA(N)

      INTEGER LWA, NSCAL
      PARAMETER(LWA = NCONT * (3 * NCONT + 13) / 2)
      PARAMETER(NSCAL = 2)

      REAL*8 XTOL, SCALFAC, OUTTOL
      PARAMETER(XTOL = 0.0000000001)
      PARAMETER(SCALFAC = 10.0)
      PARAMETER(OUTTOL  = 1.0E-10)

      INTEGER J, K, IFAIL, NEGFLAG, MARKSOL

      REAL*8 HLMIN(N), HLMAX(N), HLOPT(N)
      REAL*8 ETAMIN(N), ETAMAX(N), ETAOPT(N)
      REAL*8 XCONT(NCONT), FUNCTS(NCONT)
      REAL*8 WA(LWA)
      REAL*8 FSUMSQ

C----------------------------------------------------------------------
c      REAL*8 EPCHECK(N)
C----------------------------------------------------------------------

      EXTERNAL FUNC1

C======================================================================

      DO K = 1, N
         ARESP(K)  = RWA(K)
         ETAMIN(K) = ETA(K)
         ETAMAX(K) = ETA(K)
         HLMIN(K)  = HL(K)
         HLMAX(K)  = HL(K)
      END DO

!----------------------------------------------------------------------
! Test output of combi IRF.
!----------------------------------------------------------------------
c
c      WRITE(*,*)
c      WRITE(*,*) 'roots'
c      WRITE(*,*)
c      WRITE(*,*) 'RCT'
c      WRITE(*,'(3F16.8)') (RWT(K), K=1,3)
c
c      WRITE(*,*)
c      WRITE(*,*) 'RCA, SUM'
c      WRITE(*,'(5F16.8)') (RWA(K), K=1,4), RWA(1)+RWA(2)+RWA(3)+RWA(4)
c
c      WRITE(*,*)
c
c      write(*,*) 'HL'
c      write(*,*) HL(1), HL(2), HL(3), HL(4)
c      write(*,*) 'ETA'
c      write(*,*) ETA(1), ETA(2), ETA(3), ETA(4)
c      write(*,*) 'c2'
c
!----------------------------------------------------------------------

      DO J = 1, NSCAL
         ETAMIN(2)  = ETAMIN(2)  / SCALFAC
         ETAMAX(2)  = ETAMAX(2)  * SCALFAC
         DO K = 3, N
            ETAMIN(K) = ETAMIN(K) / SCALFAC
            HLMIN(K)  = HLMIN(K)  / SCALFAC
            ETAMAX(K) = ETAMAX(K) * SCALFAC
            HLMAX(K)  = HLMAX(K)  * SCALFAC
!----------------------------------------------------------------------
c           WRITE(*,*) '   HLMIN(', K, ') = ',
c     .               HLMIN(K), '   HLMAX(', K, ') = ', HLMAX(K)
!----------------------------------------------------------------------
         END DO
      END DO

      DO K = 1, 2*N-1
         WEIGHT(K) = 1.0
      END DO
      WEIGHT(N) = 0.0

C----------------------------------------------------------------------
C Give inverse time constants into COMMON vector TAURINV.
C----------------------------------------------------------------------

      DO K = 1, N-1
         TAURINV(K) = 1.0 / RWT(K)
      ENDDO
      TAURINV(N) = 0.0

C======================================================================
C Root seeking loop: 2N-2 fold nesting (scan 2N-2 parameters).
C At the end of each turn of its loop, each parameter is multiplied
C by 10 until it gets bigger than its maximal value.
C======================================================================

      HL(1) = HLMIN(1)
      ETA(1)    = ETAMIN(1)

      IF(HLMAX(4).GT.(HL(1) * (1.0 / ARESP(4) - 1.0))) THEN
         HLMAX(4) = HL(1) * (1.0 / ARESP(4) - 1.0)
      END IF

      IF(HLMAX(3).GT.(HL(1) * (1.0 / ARESP(4) - 1.0))) THEN
         HLMAX(3) = HL(1) * (1.0 / ARESP(4) - 1.0)
      END IF

C----------------------------------------------------------------------
C Check: output 1st-guess grid.
C----------------------------------------------------------------------
c
c      WRITE(*,*)
c      WRITE(*,'(A12,F20.6,A12,F20.6)') '  SCALFAC = ', SCALFAC,
c     .           '   OUTTOL = ', OUTTOL
c
c      WRITE(*,'(A12,F20.6,A12,F20.6)') '   HL1MIN = ', HLMIN(1),
c     .           '   HL1MAX = ', HLMAX(1)
c
c      WRITE(*,'(A12,F20.6,A12,F20.6)') '   HL3MIN = ', HLMIN(3),
c     .           '   HL3MAX = ', HLMAX(3)
c
c      WRITE(*,'(A12,F20.6,A12,F20.6)') '   HL4MIN = ', HLMIN(4),
c     .           '   HL4MAX = ', HLMAX(4)
c
c      WRITE(*,'(A12,F20.6,A12,F20.6)') '  ETA2MIN = ', ETAMIN(2),
c     .           '  ETA2MAX = ', ETAMAX(2)
c
c      WRITE(*,'(A12,F20.6,A12,F20.6)') '  ETA3MIN = ', ETAMIN(3),
c     .           '  ETA3MAX = ', ETAMAX(3)
c
c      WRITE(*,'(A12,F20.6,A12,F20.6)') '  ETA4MIN = ', ETAMIN(4),
c     .           '  ETA4MAX = ', ETAMAX(4)
c
c      WRITE(*,*)
c      WRITE(*,*)
c
C----------------------------------------------------------------------
C Start loops.
C----------------------------------------------------------------------

      MARKSOL = 0

      ETA(2) = ETAMIN(2)
      DO WHILE ((ETA(2).LE.ETAMAX(2)+XTOL).AND.MARKSOL.EQ.0)

      ETA(3) = ETAMIN(3)
      DO WHILE ((ETA(3).LE.ETAMAX(3)+XTOL).AND.MARKSOL.EQ.0)

      ETA(4) = ETAMIN(4)
      DO WHILE ((ETA(4).LE.ETAMAX(4)+XTOL).AND.MARKSOL.EQ.0)

      HL(4) = HLMIN(4)
      DO WHILE ((HL(4).LT.HLMAX(4)+XTOL).AND.MARKSOL.EQ.0)

C----------------------------------------------------------------------
C Constrain sum of hlayers to h1 (1/a4 - 1).
C----------------------------------------------------------------------

      IF(HLMAX(3).GT.(HL(1) * (1.0/ARESP(4) - 1.0) - HL(4)))
     .   THEN
         HLMAX(3) = HL(1) * (1.0 / ARESP(4) - 1.0) - HL(4)
      END IF

      HL(3) = HLMIN(3)
      DO WHILE ((HL(3).LT.HLMAX(3)+XTOL).AND.MARKSOL.EQ.0)

      HL(2) = HL(1) * (1.0/ARESP(4) - 1.0) -
     .            (HL(3) + HL(4))

C----------------------------------------------------------------------
C Initialize by first-guess the vector of control variables:
C XCONT(1,2): Layer thicknesses 3,4.
C XCONT(3,4,5): Diffusion coeffs 2...4,
C----------------------------------------------------------------------

      DO K = 2,N
         XCONT(K+N-3) = ETA(K)
      END DO

      DO K = 1, N-2
         XCONT(K)     = HL(K+2)
      END DO

      HL1 = HL(1)

C======================================================================
C Seek roots, output result.
C======================================================================

      IFAIL = 1
      CALL C05NBF(FUNC1, NCONT, XCONT, FUNCTS, XTOL, WA, LWA, IFAIL)

C----------------------------------------------------------------------
C Check whether all diff coeffs & layer thicknesses are positive.
C----------------------------------------------------------------------

      NEGFLAG = 0
      DO K = 1, 2*N-3
         IF(XCONT(K).LT.0.0) NEGFLAG = 1
      END DO

C----------------------------------------------------------------------
C Get final values of the layer depths and diffusion coefficients.
C----------------------------------------------------------------------

      ETAOPT(1) = ETA(1)

      DO K = 2, N
         ETAOPT(K) = XCONT(K+N-3)
      ENDDO

      DO K = 1, N-2
         HLOPT(K+2)  = XCONT(K)
      END DO

      HLOPT(1)  = HL(1)
      HLOPT(2)  = HLOPT(1) * (1.0/ARESP(4) - 1.0) -
     .             (HLOPT(3) + HLOPT(4))

C----------------------------------------------------------------------
C Compute cost function as sum of squares.
C----------------------------------------------------------------------

      FSUMSQ = 0.0
      DO K = 1, NCONT
         FSUMSQ = FSUMSQ + FUNCTS(K) * FUNCTS(K)
      ENDDO

C----------------------------------------------------------------------
C If terminated because of complex eigenvalues: error message only.
C----------------------------------------------------------------------

      IF((FSUMSQ.LE.OUTTOL).AND.(NEGFLAG.EQ.0)) THEN
         IF (IFAIL.EQ.-1) THEN
            WRITE(*,*) '#'
            WRITE(*,*) 'System matrix is approximately singular.'
            WRITE(*,*) 'ABNORMAL EXIT from NAG Library routine F04ATF:'
            WRITE(*,*) 'No eigenvalues/vectors found.'
            WRITE(*,*) '#'
         ELSE IF (IFAIL.EQ.-2) THEN
            WRITE(*,*) '#'
            WRITE(*,*) 'System matrix is too ill-conditioned.'
            WRITE(*,*) 'ABNORMAL EXIT from NAG Library routine F04ATF:'
            WRITE(*,*) 'No eigenvalues/vectors found.'
            WRITE(*,*) '#'
         ELSE IF (IFAIL.EQ.-3) THEN
            WRITE(*,*) '#'
            WRITE(*,*) 'Bad dimensions for system matrix or workspace.'
            WRITE(*,*) 'ABNORMAL EXIT from NAG Library routine F04ATF:'
            WRITE(*,*) 'No eigenvalues/vectors found.'
            WRITE(*,*) '#'
         ELSE IF (IFAIL.EQ.-5) THEN
            WRITE(*,*)
            WRITE(*,*) 'Found complex eigenvalues.'
            WRITE(*,*)
         ELSE
            MARKSOL = 1

C======================================================================
C Check: transform pulse back from eigenvector coordinates into box
C contents to check the precision.
C----------------------------------------------------------------------
C
C            DO J = 1, N
C               EPCHECK(J) = 0.0
C               DO K = 1, N
C                  EPCHECK(J) = EPCHECK(J) + RPULSE(K) * CEIGR(J,K)
C               ENDDO
C            ENDDO
C
C----------------------------------------------------------------------
C Check: output results for last point.
C----------------------------------------------------------------------
C
C            CALL MKMOUT(N, NCONT, ARESP, TAUR,
C     .           BRESP, LAMBDAR, LAMBDAI, FSUMSQ,
C     .           MSYST, CEIGR, CEIGI, RPULSE, EPCHECK,
C     .           HLOPT, ETAOPT)
C
C            WRITE(*,*) ('#', K=1,70)
C
C======================================================================

         END IF
      END IF

C----------------------------------------------------------------------
C Close all parameter scanning loops.
C----------------------------------------------------------------------

      HL(3) = SCALFAC * HL(3)
      END DO
      HL(4) = SCALFAC * HL(4)
      END DO

      ETA(4) = SCALFAC * ETA(4)
      END DO
      ETA(3) = SCALFAC * ETA(3)
      END DO
      ETA(2) = SCALFAC * ETA(2)
      END DO

C======================================================================
C End of the space gridding loop.
C======================================================================

      DO K = 1,N
         HL(K) = HLOPT(K)
         ETA(K) = ETAOPT(K)
      END DO

C======================================================================
C Check: transform pulse back from eigenvector coordinates into box
C contents to check the precision.
C----------------------------------------------------------------------
c
c            DO J = 1, N
c               EPCHECK(J) = 0.0
c               DO K = 1, N
c                  EPCHECK(J) = EPCHECK(J) + RPULSE(K) * CEIGR(J,K)
c               ENDDO
c            ENDDO
c
C----------------------------------------------------------------------
C Check: output results for last point.
C----------------------------------------------------------------------
c
c            CALL MKMOUT(N, NCONT, RWA, RWT,
c     .           BRESP, LAMBDAR, LAMBDAI, FSUMSQ,
c     .           MSYST, CEIGR, CEIGI, RPULSE, EPCHECK,
c     .           HL, ETA)
c
c            WRITE(*,*) ('#', K=1,70)
c
C======================================================================

      END

C######################################################################

      SUBROUTINE EIGSORT(N, EIGVAL, EIGVEC, WKSP, IPLACE)
C----------------------------------------------------------------------
C
C Purpose:
C --------
C Given the eigenvalues & eigenvectors of a matrix, this routine
C orders them according to the size of the eigenvalues so that the
C biggest e:value & the corresp. e:vector is put into the 1st place.
C
C Interface:
C ----------
C     CALL EIGSORT(N, EIGVAL, EIGVEC, WKSP, IPLACE)
C
C Input arguments:
C	N	The number & dimension of eigenvectors.
C
C In- & output (output in sorted order):
C	EIGVAL	Vec containing the eigenvalues.
C	EIGVEC	Mat containing the eigenvectors. 2nd index is the
C		 number of the e:vector, 1st index counts its components.
C
C Dummy arguments (will be overwritten by this routine):
C	WKSP,
C	IPLACE	Workspace vectors.
C
C Externals:
C ----------
C No externals.
C
C Author:
C -------
C G.Hooss, MPI Meteo, June 1997.
C----------------------------------------------------------------------

      INTEGER N, I, J, K
      INTEGER IPLACE(N)
      REAL*8 EIGVEC(N,N), EIGVAL(N), WKSP(N), LAMEXC

C----------------------------------------------------------------------
C Get place where to put each eigenvalue:
C How many others are bigger than lambda_j? If k are bigger, than
C No.j is the k+1 -biggest.
C----------------------------------------------------------------------

      DO J = 1, N
         IPLACE(J) = 1
         DO I = 1, N
            IF(EIGVAL(I).GT.EIGVAL(J)) IPLACE(J) = IPLACE(J) + 1
         END DO
      END DO

C----------------------------------------------------------------------
C Put in the right place:
C 1. the eigenvalues,
C 2. the eigenvectors,
C 3. the places where to put.
C----------------------------------------------------------------------

      DO J = 1, N
         LAMEXC = EIGVAL(IPLACE(J))
         EIGVAL(IPLACE(J)) = EIGVAL(J)
         EIGVAL(J) = LAMEXC

         DO I = 1, N
            WKSP(I) = EIGVEC(I, IPLACE(J))
            EIGVEC(I, IPLACE(J)) = EIGVEC(I,J)
            EIGVEC(I,J) = WKSP(I)
         END DO

         K = IPLACE(IPLACE(J))
         IPLACE(IPLACE(J)) = IPLACE(J)
         IPLACE(J) = K

      END DO

      RETURN
      END

C######################################################################

      SUBROUTINE FUNC1(NN, XC, FVECC, IFL)
C----------------------------------------------------------------------
C
C Purpose:
C --------
C User-supplied subroutine to be called from NAG optimization routine
C E04FDF. Computes vector of functions of which the sum of squares is
C to be minimized.
C
C Actions performed by this routine:
C
C Build system matrix from model parameters.
C Get eigenvalues & eigenvectors by NAG routine.
C
C Check whether there are complex eigenvalues/vectors. If yes, set
C the IFL flag to -5 to tell the calling routine that XC was illegal.
C
C Transform pulse into uppermost box to eigenvector coordinates by
C call to a NAG routine that solves the linear equation system
C (( CEIGR )) ( RPULSE ) = ( EPULSE, 0, 0, 0 ).
C
C Version 5: 2 hlayer constraints reduce number of indep var:s to 5.
C
C Version 6: Cost function with relative differences instead of absolute.
C
C Build cost function terms.
C
C
C Interface:
C ----------
C     CALL LSFUN1(NN, NN, XC, FVECC)
C
C Input arguments:
C	NN	Number of control variables & functions.
C	XC	Vector of control variables.
C
C Output argument:
C	FVECC	Vector of functions.
C	IFL	Flag that can be set to negative values to indicate
C		 that the calling routine is to be stopped.
C
C Externals:
C ----------
C No externals.
C
C Author:
C -------
C G.Hooss, MPI Meteo HH, June 1997.
C----------------------------------------------------------------------

      IMPLICIT NONE

#include "NICOroot.h"

      INTEGER NN, IFL
      REAL*8 XC(NN), FVECC(NN)

      REAL*8 EPSM, PULSE, PULSINV
      PARAMETER(EPSM    = 0.0000001)
      PARAMETER(PULSE   = 1.0)
      PARAMETER(PULSINV = 1.0)

      INTEGER I, J, K
      INTEGER IFAIL, INTGER(N), IWKSP(N)

      REAL*8 AA(N,N)
      REAL*8 ETADIF(N), HL(N), HLINV(N)
      REAL*8 WKSP1(N), WKSP2(N), EPULSE(N)

C----------------------------------------------------------------------
C Local fixed parameters:
C
C	EPSM	Threshold for complexity check of eigenvalues.
C	PULSE	Carbon input pulse for calibration.
C	PULSINV	Inverse of calibration pulse.
C
C Local variables:
C	I,J,K	Counting indices.
C	IFAIL	NAG error indicator.
C	INTGER	NAG output vec (No. of iterations for each
C		 eigenvalue).
C	AA	Workspace matrix for NAG routine, also used as
C		 safe copy of eigenvector matrix.
C	ETADIF	Diffusion coefficients, taken from control var vec.
C	HL	Vec of layer thicknesses.
C	HLINV	Vec of inverse layer thicknesses.
C	WKSP1,2	Workspace vectors for NAG routine.
C	IWKSP	Workspace vector for sorting of eigenvectors.
C	EPULSE	Vec with calibration pulse in its 1st component.
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C Check number of variables.
C----------------------------------------------------------------------

      IF (NN.NE.NCONT) THEN
         WRITE(*,*)
     .       'ERROR in FUNC1: Number of control variables is ', NN
         STOP
      ENDIF

C----------------------------------------------------------------------
C The control variables are organized as follows:
C XC(1,2) are layer depths HLINV(3,4), and
C XC(3,4,5) are the diffusion coefficients ETADIF(2,3,4).
C The 1st box' thickness is COMMON with the main prog.
C The 1st element of the ETADIF vector is irrelevant & ignored.
C----------------------------------------------------------------------

      HL(1) = HL1

      DO K = 3, N
         HL(K) = XC(K-2)
      END DO

      DO K = 2, N
         ETADIF(K) = XC(K+1)
      ENDDO

C----------------------------------------------------------------------
C Constrain sum of hlayers to h1 (1/a4 - 1). 
C----------------------------------------------------------------------

      HL(2) = HL(1) * (1.0/ARESP(4) - 1.0) - (HL(3) + HL(4))

      DO K = 1,N
         HLINV(K) = 1.0 / HL(K)
      END DO

C----------------------------------------------------------------------
C Build system matrix.
C----------------------------------------------------------------------

      MSYST(1,1) =   ETADIF(2) * HLINV(1)
      MSYST(1,2) = - ETADIF(2) * HLINV(2)
      MSYST(1,3) =   0.0
      MSYST(1,4) =   0.0

      MSYST(2,1) = - ETADIF(2) * HLINV(1)
      MSYST(2,2) =  (ETADIF(2)+ETADIF(3)) * HLINV(2)
      MSYST(2,3) = - ETADIF(3) * HLINV(3)
      MSYST(2,4) =  0.0

      MSYST(3,1) =  0.0
      MSYST(3,2) = - ETADIF(3) * HLINV(2)
      MSYST(3,3) =  (ETADIF(3)+ETADIF(4)) * HLINV(3)
      MSYST(3,4) = - ETADIF(4) * HLINV(4)

      MSYST(4,1) =  0.0
      MSYST(4,2) =  0.0
      MSYST(4,3) = - ETADIF(4) * HLINV(3)
      MSYST(4,4) =   ETADIF(4) * HLINV(4)

C----------------------------------------------------------------------
C Call NAG routine to compute eigenvalues & eigenvectors.
C----------------------------------------------------------------------

      IFAIL = 0

      DO I=0,N
         DO J=0,N
            AA(I,J) = MSYST(I,J)
         ENDDO
      ENDDO

      CALL F02AGF(AA, N, N,
     .            LAMBDAR, LAMBDAI, CEIGR, N, CEIGI, N,
     .            INTGER, IFAIL)

C----------------------------------------------------------------------
C Ceck whether there are complex eigenvalues.
C----------------------------------------------------------------------

      DO J = 1,N
         IF ((LAMBDAI(J)*LAMBDAI(J)).GT.EPSM) THEN
            IFL = -5
            RETURN
         ENDIF
      ENDDO

C----------------------------------------------------------------------
C Sort eigenvectors & eigenvalues so that biggest eigenvalue is 1st.
C----------------------------------------------------------------------

      CALL EIGSORT(N, LAMBDAR, CEIGR, WKSP1, IWKSP)

C----------------------------------------------------------------------
C Give carbon input pulse in box content coordinates.
C----------------------------------------------------------------------

      EPULSE(1) = PULSE
      EPULSE(2) = 0.0
      EPULSE(3) = 0.0
      EPULSE(4) = 0.0

C----------------------------------------------------------------------
C Compute impulse components in eigenvector coordinates by NAG routine
C solving the linear equation system CEIGR * RPULSE = EPULSE
C----------------------------------------------------------------------

      IFAIL = 1
      CALL F04ATF(CEIGR, N, EPULSE, N, RPULSE,
     .            AA, N, WKSP1, WKSP2, IFAIL)
      IF(IFAIL.EQ.1) THEN
         IFL = -1
         RETURN
      ELSEIF(IFAIL.EQ.2) THEN
         IFL = -2
         RETURN
      ELSEIF(IFAIL.EQ.3) THEN
         IFL = -3
         RETURN
      ENDIF

C----------------------------------------------------------------------
C Compute response coefficients.
C----------------------------------------------------------------------

      DO K = 1, N
         BRESP(K)   = PULSINV * RPULSE(K) * CEIGR(1,K)
      ENDDO

C----------------------------------------------------------------------
C Functions of which the zeroes are to be found:
C 1,2 from response coefficients; 3,4,5 from inverse time constants.
C Weights No. 3, 4, & 8 are ignored.
C----------------------------------------------------------------------

      DO K = 1, N-2
         FVECC(K)     = WEIGHT(K) * (BRESP(K) - ARESP(K)) / ARESP(K)
      END DO

      DO K = 1, N-1
         FVECC(K+N-2) = WEIGHT(K+N) * (LAMBDAR(K) - TAURINV(K)) /
     .                         TAURINV(K)
      END DO

C----------------------------------------------------------------------

      RETURN
      END

C######################################################################

      SUBROUTINE MKMOUT(N, NRES,
     .                  ARESP, TAU,
     .                  BRESP, LAMBDAR, LAMBDAI, FSUMSQ,
     .                  MSYST, CEIGR, CEIGI,
     .                  RPULSE, EPCHECK, HLOPT, ETAOPT)
C----------------------------------------------------------------------
C Purpose:
C --------
C Write final output to stdout and to check file.
C
C Interface:
C ----------
C     CALL MKMOUT(N, NRES, ARESP, TAU,
C    .           BRESP, LAMBDAR, LAMBDAI, FSUMSQ,
C    .           MSYST, CEIGR, CEIGI, RPULSE, EPCHECK, HLOPT, ETAOPT)
C
C Input arguments:
C       N, NRES
C	ARESP	Coefficients from the original response function.
C	BRESP	Optimized coefficients for model response function.
C	TAU	Time constants from the original response function.
C	MSYST	Model's system matrix.
C	LAMBDAR, Real & imaginary parts of the eigenvalues.
C	 LAMBDAI
C	CEIGR,	 Real & imaginary parts of the eigenvectors (columns).
C	 CEIGI
C	HLAYER	1st guess layer thicknesses.
C	ETA	1st guess diffusion coefficients.
C	HLOPT	Optimized layer thicknesses.
C	ETAOPT	Optimized diffusion coefficients.
C	FSUMSQ	Value of the cost function.
C	RPULSE	Calibration pulse in eigenvector coordinates.
C	EPCHECK Calibration pulse in box content coordinates.
C
C No output arguments.
C
C Dummy arguments (these will be unchanged by the routine):
C
C No externals.
C -------------
C
C Author:
C -------
C G.Hooss, MPI Me 1997-06-10
C----------------------------------------------------------------------

      IMPLICIT NONE

      INTEGER N, NRES, I, J, K
      REAL*8 ARESP(N), TAU(N), HLOPT(N), ETAOPT(N)
      REAL*8 BRESP(N), LAMBDAR(N), LAMBDAI(N), RPULSE(N), EPCHECK(N)
      REAL*8 MSYST(N,N), CEIGR(N,N), CEIGI(N,N)
      REAL*8 FSUMSQ
      CHARACTER*8 FMT

C----------------------------------------------------------------------
C Output format according to system size.
C----------------------------------------------------------------------

      FMT = '('//CHAR(N+ICHAR('0'))//'F16.6)'

      WRITE(*,*) '#'

C----------------------------------------------------------------------
C Output to check file: final values of the layer depths
C  and diffusion coefficients.
C----------------------------------------------------------------------

      WRITE(*,*) '#'
      WRITE(*,*) '# Hlayer & Eta (final values):'
      WRITE(*,*) '#'
      WRITE(*,FMT) (HLOPT(K), K=1,N)
      WRITE(*,FMT) (ETAOPT(K), K=1,N)
      WRITE(*,*) '#'

C----------------------------------------------------------------------
C Output to check file: system matrix.
C----------------------------------------------------------------------

      WRITE(*,*) '# System matrix ( N = ', N, ') :'
      WRITE(*,*) '#'
      DO I = 1, N
        WRITE(*, FMT) (MSYST(I,J), J = 1,N)
      ENDDO
      WRITE(*,*) '#'

C----------------------------------------------------------------------
C Output to check file: eigenvalues & eigenvectors.
C Eigenvalue number J is in LAMBDAR(J) (real part)
C                       and LAMBDAI(J) (imaginary part).
C The eigenvector corresponding to eigenvalue No. J is in the J:th
C column of the eigenvector matrices CEIGR(I,J) (real parts)
C                                and CEIGI(I,J) (imaginary parts).
C So, for fixed column index J, the row index I counts the components
C of the J:th eigenvector.
C----------------------------------------------------------------------

      WRITE(*,*) '# Eigenvalues:'
      WRITE(*,*) '#'

      DO J = 1, N
         WRITE(*,'(A9, I02, A4, F8.4, A9, I02, A4, F8.4)')
     .           '  Lambda(', J, ') = ', LAMBDAR(J),
     .           ' ;  Linv(', J, ') = ', 1.0 / LAMBDAR(J)
      ENDDO

      WRITE(*,*) '#'
      WRITE(*,*) '# Eigenvectors:'
      WRITE(*,*) '#'

      DO I = 1, N
         WRITE(*, FMT) (CEIGR(I,J), J = 1,N)
      ENDDO

      WRITE(*,*) '#'

C----------------------------------------------------------------------
C Output to check file: pulse in eigenvector coordinates
C i.e. the fractions by which the eigenvectors contribute to the
C input pulse. Output backtransformed pulse in box content coordinates.
C----------------------------------------------------------------------

      WRITE(*,*) '# Pulse contributions by eigenvectors (from Mx=b)'
      WRITE(*,*) '#  and by boxes (backtransformed):'
      WRITE(*,*) '#'
      WRITE(*, FMT) (RPULSE(K), K=1,N)
      WRITE(*, FMT) (EPCHECK(K), K=1,N)
      WRITE(*,*) '#'

C----------------------------------------------------------------------
C Output current time constants & response coeffs.
C----------------------------------------------------------------------

      WRITE(*,*)
     .  '# Response coeffs (orig & model):'
      WRITE(*,*) '#'
      WRITE(*,'(4F16.6)') (ARESP(I), I = 1,N)
      WRITE(*,'(4F16.6)') (BRESP(I), I = 1,N)
      WRITE(*,*) '#'

      WRITE(*,*)
     .   '# Time constants; inverse eigenvalues:'
      WRITE(*,*) '#'
      WRITE(*,'(3F16.6, A12)') (TAU(I), I = 1,N-1), 'inf'
      WRITE(*,'(4F16.6)') (1.0 / LAMBDAR(I), I = 1,N)
      WRITE(*,*) '#'

C----------------------------------------------------------------------
C Output vaule of the cost function.
C----------------------------------------------------------------------

      WRITE(*,*) '# Cost function:'
      WRITE(*,*) '#'
      WRITE(*,'(E12.5)') FSUMSQ
      WRITE(*,*) '#'

C----------------------------------------------------------------------

      RETURN
      END

C--  Last line.  ------------------------------------------------------
