C----------------------------------------------------------------------

      SUBROUTINE CHEM
C----------------------------------------------------------------------
C Purpose:
C --------
C Compute DIC and Revelle buffer factor for given pCO2 and seawater
C chemistry.
C
C Interface:
C ----------
C     CALL CHEM
C All arguments are passed by the COMMON blocks in the #include files.
C
C Input (unchanged on exit):
C --------------------------
C	ALPHAS	Inverse solubility of CO2 (Henry's constant).
C	AK1	Dissociation constant of CO2.
C	AK2	Diss:const. of HCO3+
C	AKB	Diss:const. of B(OH)3
C	AKW	Diss:const. of water
C	ALK	Alkalinity
C	BTS	Total boron [B(OH)3]+[B(OH)4-]
C	PCO2	partial pressure of CO2 (ppmv of surface-pressure air).
C
C Output (changed on exit):
C ------------------------------
C	SIGC0	Total dissolved inorganic carbon (DIC). Entry value ignored.
C	XIBUF	Revelle buffer factor. Entry value ignored.
C
C Local variables:
C ----------------
C	NITER	Number of iterations in.
C	HPLUS	[H+] Entry value is used as first-guess.
C		 First guess: 1.0E-11 < HPLUS < 1.0E-07 for convergence.
C	EPSILON	Relative change in pCO2 for finite-difference est. of xibuf.
C		 Best accuracy (6 digits) has been found with EPSILON = 1.0E-07.
C	XIBUF1	Upper and
C	XIBUF2	 lower finite-difference est. of xibuf.
C
C External:
C ---------
C PPM2DIC	Compute DIC and [H+] for given pCO2.
C
C Method:
C -------
C Call PPM2DIC for several slightly different pCO2 values for finite-
C difference estimate of the buffer factor.
C
C Author:
C -------
C G.Hooss, MPI Met, Oct 1999.
C
C
C Copyright
C ---------
C (C) 2001 K.G.Hooss
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License
C as published by the Free Software Foundation; either version 2
C of the License, or (at your option) any later version.
C 
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C 
C You should have received a copy of the GNU General Public License
C along with this program; if not, write to the Free Software
C Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
C----------------------------------------------------------------------

      IMPLICIT NONE
      INTEGER NITER

#include "scparche.h"
#include "scmatbox.h"

      REAL PCO2

      REAL HPLUS, SIGMACM1, SIGMACP1, XIBUF1, XIBUF2
      REAL EPSILON

C----------------------------------------------------------------------

      PARAMETER(NITER = 40)
      PARAMETER(EPSILON = 1.0E-7)

C----------------------------------------------------------------------

      PCO2 = W0 * ALPHAA

      HPLUS = 1.0E-9

      OPEN(9, FILE='xibuf.log')

      CALL PPM2DIC(ALPHAS, AK1, AK2, AKB, AKW, ALK, BTS,
     .             (1.0-EPSILON)*PCO2, HPLUS, SIGMACM1, NITER)

      CALL PPM2DIC(ALPHAS, AK1, AK2, AKB, AKW, ALK, BTS,
     .             PCO2, HPLUS, SIGC0, NITER)

      CALL PPM2DIC(ALPHAS, AK1, AK2, AKB, AKW, ALK, BTS,
     .             (1.0+EPSILON)*PCO2, HPLUS, SIGMACP1, NITER)

      XIBUF1 = SIGC0 * EPSILON / (SIGC0   - SIGMACM1)
      XIBUF2 = SIGC0 * EPSILON / (SIGMACP1 - SIGC0)
      XIBUF  = SIGC0 * 2 * EPSILON / (SIGMACP1 - SIGMACM1)

      WRITE(9,*)
      WRITE(9,'(A8,2A16)') 'NITER', 'HPLUS', 'ALK'
      WRITE(9,'(I8,2E16.8)') NITER, HPLUS, ALK
      WRITE(9,*)
      WRITE(9,'(2A16)') 'PCO2', 'SIGC0'
      WRITE(9,'(2E16.8)') (1.0-EPSILON)*PCO2, SIGMACM1
      WRITE(9,'(2E16.8)')               PCO2, SIGC0
      WRITE(9,'(2E16.8)') (1.0+EPSILON)*PCO2, SIGMACP1
      WRITE(9,*)
      WRITE(9,*) 'EPSILON = ', EPSILON
      WRITE(9,*) 'XIBUF1  = ', XIBUF1
      WRITE(9,*) 'XIBUF   = ', XIBUF
      WRITE(9,*) 'XIBUF2  = ', XIBUF2
      WRITE(9,*)

      CLOSE(9)

      RETURN
      END

C----------------------------------------------------------------------

      SUBROUTINE PPM2DIC(ALPHAS, AK1, AK2, AKB, AKW, ALK, BTS,
     .                   PCO2, HPLUS, SIGMAC, NITER)
C----------------------------------------------------------------------
C Purpose:
C --------
C Compute total DIC concentration and Revelle buffer factor
C for given PCO2, Alkalinity, total Boron, and chem.parameters.
C
C Interface:
C ----------
C     CALL PPM2DIC(ALPHAS, AK1, AK2, AKB, AKW, ALK, BTS,
C    .                   PCO2, HPLUS, SIGMAC, NITER)
C
C Input (unchanged on exit):
C --------------------------
C	ALPHAS	Inverse solubility of CO2 (Henry's constant).
C	AK1	Dissociation constant of CO2.
C	AK2	Diss:const. of HCO3+
C	AKB	Diss:const. of B(OH)3
C	AKW	Diss:const. of water
C	ALK	Alkalinity
C	BTS	Total boron [B(OH)3]+[B(OH)4-]
C	PCO2	partial pressure of CO2 (ppmv of surface-pressure air).
C	NITER	Number of iterations.
C
C Output (changed on exit):
C ------------------------------
C	HPLUS	[H+] Entry value is used as first-guess.
C		 First guess: 1.0E-11 < HPLUS < 1.0E-07 for convergence.
C	SIGMAC	Total dissolved inorganic carbon (DIC). Entry value ignored.
C
C Method:
C -------
C At given K:s, alpha_s, SigmaB, and pCO2, solve alkalinity as a function
C of HPLUS by Newton iteration using dA/dH. Once HPLUS is found, SigmaC can
C be computed as
C SigmaC = (1 + K1/H + K1K2/H^2) * pCO2 / alpha_s
C
C No externals.
C -------------
C
C Author:
C -------
C G.Hooss, MPI Met, Oct 1999.
C----------------------------------------------------------------------

      IMPLICIT NONE
      INTEGER ITER, NITER

      REAL ALPHAS
      REAL AK1, AK2, AKB, AKBINV, AKW
      REAL ALK, AA, DADH, BTS
      REAL PCO2
      REAL HPLUS, HPLUSINV, SIGMAC

C----------------------------------------------------------------------

      AKBINV = 1.0 / AKB
      HPLUSINV = 1.0 / HPLUS
      AA = (AK1*HPLUSINV + 2*AK1*AK2*HPLUSINV*HPLUSINV)
     .     * PCO2 / ALPHAS
     .    + BTS / (1.0 + HPLUS * AKBINV) + AKW*HPLUSINV - HPLUS - ALK

      DADH = - (AK1*HPLUSINV*HPLUSINV
     .           + 4 * AK1*AK2*HPLUSINV*HPLUSINV*HPLUSINV)
     .         * PCO2 / ALPHAS
     .       - BTS * AKBINV
     .         / (1.0 + HPLUS * AKBINV) * (1.0 + HPLUS * AKBINV)
     .       - AKW * HPLUSINV * HPLUSINV - 1.0

      ITER = 0

      WRITE(9,'(A8,2A16)') 'ITER', 'HPLUS', 'AA'
      WRITE(9,'(I8,2E16.8)') ITER, HPLUS, AA

      DO ITER=1, NITER
        HPLUS = HPLUS - AA / DADH
        HPLUSINV = 1.0 / HPLUS
        AA = (AK1*HPLUSINV + 2*AK1*AK2*HPLUSINV*HPLUSINV)
     .       * PCO2 / ALPHAS
     .      + BTS / (1.0 + HPLUS * AKBINV) + AKW*HPLUSINV
     .      - HPLUS - ALK

        DADH = - (AK1*HPLUSINV*HPLUSINV
     .            + 4 * AK1*AK2*HPLUSINV*HPLUSINV*HPLUSINV)
     .           * PCO2 / ALPHAS
     .         - BTS * AKBINV
     .           / (1.0 + HPLUS * AKBINV) * (1.0 + HPLUS * AKBINV)
     .         - AKW * HPLUSINV * HPLUSINV - 1.0

        WRITE(9,'(I8,2E16.8)') ITER, HPLUS, AA
      END DO

      SIGMAC = (1.0 + AK1*HPLUSINV + AK1*AK2*HPLUSINV*HPLUSINV)
     .         * PCO2 / ALPHAS

      WRITE(9,'(A8,4A16)') 'ITER', 'HPLUS', 'AA', 'PCO2', 'SIGMAC'
      WRITE(9,'(I8,4E16.8)') ITER, HPLUS, AA, PCO2, SIGMAC

      RETURN
      END

C----------------------------------------------------------------------
