
      PROGRAM XIBUF

C----------------------------------------------------------------------
C Purpose:
C --------
C Compute unperturbed (preindustrial) CO2 partial pressure from
C given DIC concentration & chemical parameters (values from Bacastow 1981).
C The preindustrial DIC (SigmaC_0, SC0 in the program)
C had to be adjusted from 2.089E-3 mol/l (Bacastow) to 2.0809E-3 mol/l
C in order to give the correct pCO2_0 of 280 ppm (Rep.192).
C
C Variables:
C ----------
C	HH	Hydrogen ion concentration in mol/l.
C	PCO2	CO2 partial pressure (ppm).
C
C	ALK	Constant value of the alkalinity (mol/l).
C	BTS	Total borate concentration (mol/l).
C	SC0	DIC in surface layer (mol/l).
C
C	AK1, AK2, AKB
C		Dissociation constants for (bi-)carbonate & borate (mol/l).
C	AKW	Dissociation constant of water (mol/l)^2 .
C
C	ALPHAA	Conversion atmosph.C (GtC) -> atm.'partial pressure'(ppm).
C	ALPHAS	Henry's constant (inverse solubility),
C		                            [H2CO3] (mol/l) -> pCO2 (ppm).
C
C References:
C -----------
C E.Maier-Reimer: pers.comm. (1997).
C R.Bacastow: "Numerical evaluation of the evasion factor", section 3.4
C of B.Bolin (ed): "Carbon cycle modelling" (SCOPE 16, 1981).
C
C Author:
C -------
C G.Hooss; MPI Me 8:1997.
C
C Changes:
C --------
C 1999-09-27
C Compute pCO2,0 and xi for chemical constants (incl SigmaC_0) from
C HAMOCC surface grid cell [72,15] (North Atlantic deepwater formation).
C DIC and ALKALI from simulation year 48 (jan), rest from chem init (jan).
C
C
C Copyright
C ---------
C (C) 2001 K.G.Hooss
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License
C as published by the Free Software Foundation; either version 2
C of the License, or (at your option) any later version.
C 
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C 
C You should have received a copy of the GNU General Public License
C along with this program; if not, write to the Free Software
C Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
C----------------------------------------------------------------

      IMPLICIT NONE

      INTEGER ISC, NSC, ITER, NITER

      PARAMETER(NSC   = 200)
      PARAMETER(NITER =  10)

      REAL HH, PCO2_0, SigmaC
      REAL ALK, BTS, W0, AK1, AK2, AKB, AKW
      REAL ALPHAA, ALPHAS, AINVA
      REAL AA, DADH, H1, DENOM, SC, CFAC, XI

      REAL PCO2(NSC), SC(NSC)

C----------------------------------------------------------------------
C Input.
C----------------------------------------------------------------------

      READ(*,*) ALPHAA
      READ(*,*) ALPHAS
      READ(*,*) AK1
      READ(*,*) AK2
      READ(*,*) AKB
      READ(*,*) AKW

      READ(*,*) ALK
      READ(*,*) BTS

      READ(*,*) SigmaC
      READ(*,*) CFAC

      AINVA = 1.0 / ALPHAA

C----------------------------------------------------------------------
C Newton iteration for solution of the CO2 chemistry for given
C Alk and Sigma C. Do it once for initial guess of unperturbed Sigma C.
C----------------------------------------------------------------------

      HH   = 5.0E-9

      H1   = HH
      ITER = 0
      WRITE(*,*)
      WRITE(*,'(A4,4A16)') 'ITER', 'ALK', 'AA', 'HH', 'HH-H1'

      DENOM = HH*HH + AK1*HH + AK1*AK2

      AA   = SigmaC * (AK1*HH + 2*AK1*AK2) / DENOM +
     .       AKB*BTS / (AKB+HH) + AKW/HH - HH - ALK

      DADH = SigmaC*AK1 / DENOM -
     .      SigmaC * (AK1*HH + 2*AK1*AK2) *
     .           (2*HH + AK1) / (DENOM * DENOM) -
     .      AKB*BTS/((AKB+HH)*(AKB+HH)) - AKW/(HH*HH) - 1

      WRITE(*,'(I4,4E16.6)') ITER, ALK, AA, HH, HH-H1

      DO ITER = 1,NITER
         H1    = HH
         HH    = HH - AA / DADH

         IF (HH.LT.0.0) HH = 0.0 - HH
         DO WHILE (HH.GT.1.0E-4)
           HH = 0.01 * HH
         END DO

         DENOM = HH*HH + AK1*HH + AK1*AK2

         AA    = SigmaC * (AK1*HH + 2*AK1*AK2) / DENOM +
     .           AKB*BTS / (AKB+HH) + AKW/HH - HH - ALK

         DADH  = SigmaC*AK1 / DENOM -
     .           SigmaC * (AK1*HH + 2*AK1*AK2) *
     .           (2*HH + AK1) / (DENOM * DENOM) -
     .           AKB*BTS/((AKB+HH)*(AKB+HH)) - AKW/(HH*HH) - 1

         WRITE(*,'(I4,4E16.6)') ITER, ALK, AA, HH, HH-H1
      END DO

      PCO2_0 = ALPHAS*SigmaC*HH*HH / DENOM
      W0 = PCO2_0 * AINVA

      WRITE(*,*)
      WRITE(*,'(A4,A20,2A16,A18)') '#isc', 'DIC',
     .                         'pCO2/ppm', 'w0/GtC', 'xi'
      WRITE(*,'(A1,I3,E20.10,2E16.6)')
     .          '#', ISC, SigmaC, PCO2_0, w0


C----------------------------------------------------------------------
C Try different preindustrial DIC concentrations to get correct
C preindustrial pCO2 and buffer/evasion factor.
C----------------------------------------------------------------------

      DO ISC = 1, NSC
         SC(ISC) = ISC * CFAC * SigmaC + SigmaC

C----------------------------------------------------------------------
C Newton iteration for solution of the CO2 chemistry for given
C Alk and C1.
C----------------------------------------------------------------------

         H1   = HH
         ITER = 0
c         WRITE(*,*)
c         WRITE(*,'(A2,4A16)') 'it','alk','aa','h','del h'

         DENOM = HH*HH + AK1*HH + AK1*AK2

         AA   = SC(ISC) * (AK1*HH + 2*AK1*AK2) / DENOM +
     .          AKB*BTS / (AKB+HH) + AKW/HH - HH - ALK

         DADH = SC(ISC)*AK1 / DENOM -
     .          SC(ISC) * (AK1*HH + 2*AK1*AK2) *
     .                (2*HH + AK1) / (DENOM * DENOM) -
     .          AKB*BTS/((AKB+HH)*(AKB+HH)) - AKW/(HH*HH) - 1

c         WRITE(*,'(I2,4E16.6)') ITER, ALK, AA, HH, HH-H1

         DO ITER = 1,NITER
            H1    = HH
            HH    = HH - AA / DADH

            IF (HH.LE.0.0) HH = 0.0 - HH
            IF (HH.GT.1.0E-4) HH = 0.01 * HH

            DENOM = HH*HH + AK1*HH + AK1*AK2

            AA    = SC(ISC) * (AK1*HH + 2*AK1*AK2) / DENOM +
     .              AKB*BTS / (AKB+HH) + AKW/HH - HH - ALK

            DADH  = SC(ISC)*AK1 / DENOM -
     .              SC(ISC) * (AK1*HH + 2*AK1*AK2) *
     .                    (2*HH + AK1) / (DENOM * DENOM) -
     .              AKB*BTS/((AKB+HH)*(AKB+HH)) - AKW/(HH*HH) - 1

c            WRITE(*,'(I2,4E16.6)') ITER, ALK, AA, HH, HH-H1
         END DO

         PCO2(ISC) = ALPHAS*SC(ISC)*HH*HH / DENOM
         W0 = PCO2(ISC) * AINVA

         XI = (PCO2(ISC) - PCO2_0) * SigmaC
     .            / (PCO2_0 * (SC(ISC) - SigmaC))

c         WRITE(*,*)
c         WRITE(*,'(A4,A16,3A10)') 'isc', 'DIC',
c     .                        'pCO2/ppm', 'w0/GtC', 'XI'
         WRITE(*,'(I4,E20.10,2E16.6,E18.8)') ISC,
     .                      SC(ISC), PCO2(ISC), w0, XI

      END DO

      END

C----------------------------------------------------------------------
