C######################################################################
C LICCS
C======================================================================
C
C Name:
C -----
C 
C Linear
C Impulse-response representation of the coupled
C Carbon cycle-
C Climate
C System
C 
C
C Method:
C -------
C Convolution integral
C (see Meier-Reimer & Hasselmann 1987, Hasselmann & al 1997).
C 
C
C I/O:
C ----
C
C In:  climate module paramters
C      emission scenario
C       (time series of annual global CO2 emissions)
C
C Out: concentration,
C      global-mean T2m,
C      cloud cover (clo),
C      precipitation (pre) and
C      sea level change (sea) time series.
C
C
C Compilation:
C ------------
C
C NLIRFMOD.F can be compiled / linked
C  with NICAOS.F instead of NICCS.F or LICCS.F.
C NLIRFMOD calls CLIM and gets the one linked by the compiler.
C
C 
C Author:
C -------
C S.Hasselmann 1996,
C G.Hooss & U.Kilian 1997,
C G.Hooss 2001
C all at the Max Planck Institute for Meteorology
C
C
C Copyright
C ---------
C (C) 2001 K.G.Hooss
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License
C as published by the Free Software Foundation; either version 2
C of the License, or (at your option) any later version.
C 
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C 
C You should have received a copy of the GNU General Public License
C along with this program; if not, write to the Free Software
C Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
C######################################################################

       SUBROUTINE CONVOL00(AR1, AR2, NCON, DT, SUM)
C-------------------------------------------------------------------
C Purpose:
C --------
C Compute convolution integral.
C
C Interface:
C ----------
C     CALL CONVOL00(AR1, AR2, NCON, DT, SUM)
C
C Input arguments:
C	AR1	Data vector 1, forward-integrated.
C	AR2	Data vector 2, backward-integrated.
C	NCON	Number of time steps. All vectors are expected to
C		 have dimensions [0:NCON].
C	DT	Length of time step.
C
C Output arguments:
C	SUM	Vector of values of the convolution integral.
C
C Externals:
C ----------
C No externals.
C
C Authors:
C --------
C S.Hasselmann, MPI Me HH, Apr 1995,
C U.Kilian & G.Hooss, MPI Me HH, Dec 1996.
C G.Hooss, MPI Me HH, Jan 1997.
C-------------------------------------------------------------------

      IMPLICIT NONE
      REAL    AR1(0:NCON), AR2(0:NCON), SUM(0:NCON), DT
      INTEGER NCON, K, J

      SUM(0)  = 0.0

      DO J = 1,NCON
        SUM(J)  = 0.0
        DO K=1,J-1
          SUM(J) = SUM(J) + AR1(K)*AR2(J-K)
        END DO
        SUM(J) = (SUM(J)+0.5*(AR2(0)*AR1(J)+AR1(0)*AR2(J)))*DT
      END DO

      RETURN
      END

C######################################################################

      SUBROUTINE CONVCARC(EL, W)
C-------------------------------------------------------------------
C Purpose:
C --------
C Compute atmospheric CO2 concentrations W from emissions EL
C by convolving with impulse response function RW.
C
C Interface:
C ----------
C     CALL CONVCARC(EL, W)
C
C Input arguments:
C	EL	Emissions.
C
C Output argument:
C	W	Concentrations.
C
C The argument vectors are expected to have dimensions (0:NTAU).
C
C Externals:
C ----------
C	CONVOL00		compute e(t) -> w(t) imp resp model.
C
C Authors:
C --------
C S.Hasselmann, MPI Me HH, Apr 1995.
C G.Hooss, MPI Me HH, Dec 1996.
C------------------------------------------------------------------

      IMPLICIT NONE
#include "spartime.h"
#include "scarrirf.h"
#include "scparche.h"
      REAL EL(0:NTAU), W(0:NTAU)
      INTEGER NT

      CALL CONVOL00(EL, RW, NTAU, DT, W)

      DO NT = 0,NTAU
        W(NT) = W0 + W(NT)
      END DO

      RETURN
      END

C######################################################################

      SUBROUTINE CONVCLIC(W, PCT, PCC, PCP, PCS)
C-------------------------------------------------------------------
C Purpose:
C --------
C Compute climate response to atmospheric CO2 concentrations W
C by convolving with impulse response functions (RPCT, ...)
C
C Interface:
C ----------
C     CALL CONVCLIC(W, PCT, PCC, PCP, PCS)
C
C Input arguments:
C	W	Concentrations.
C
C Output argument:
C	PCT	Global mean (norm.pc1) T2m.
C	PCC	Global mean (norm.pc1) Cloud cover.
C	PCP	Global mean (norm.pc1) Precip.
C	PCS	Global mean (norm.pc1) Sea level.
C
C The argument vectors are expected to have dimensions (0:NTAU).
C
C Externals:
C ----------
C	CONVOL00	w(t) -> clic(t) imp resp model.
C
C Author:
C -------
C G.Hooss, MPI Me HH, Dec 1996.
C------------------------------------------------------------------

      IMPLICIT NONE
#include "spartime.h"
#include "scarrirf.h"
#include "scparche.h"
      REAL W(0:NTAU),
     .     PCT(0:NTAU), PCC(0:NTAU), PCP(0:NTAU), PCS(0:NTAU)

      REAL DT2INV, DWDT(0:NTAU)
      REAL W0INV
      INTEGER NT

C------------------------------------------------------------------
C 1. Time derivative DWDT of concentrations W.
C------------------------------------------------------------------

      DT2INV = 0.5/DT
      W0INV  = 1.0/W0

      DO NT = 1, NTAU-1
        DWDT(NT) =   DT2INV * (W(NT+1) - W(NT-1))
      ENDDO

      DWDT(0)    = 2*DT2INV * (W(1)    - W(0))
      DWDT(NTAU) = 2*DT2INV * (W(NTAU) - W(NTAU-1))

C------------------------------------------------------------------
C 2. Compute temperatures from DWDT and IRFs.
C------------------------------------------------------------------
      CALL CONVOL00(DWDT, RPCT, NTAU, DT, PCT)
      CALL CONVOL00(DWDT, RPCC, NTAU, DT, PCC)
      CALL CONVOL00(DWDT, RPCP, NTAU, DT, PCP)
      CALL CONVOL00(DWDT, RPCS, NTAU, DT, PCS)

C------------------------------------------------------------------
C 3. Original RT contained factor 1/w0.
C------------------------------------------------------------------
      DO NT = 0, NTAU
        PCT(NT) = W0INV * PCT(NT)
        PCC(NT) = W0INV * PCC(NT)
        PCP(NT) = W0INV * PCP(NT)
        PCS(NT) = W0INV * PCS(NT)
      END DO

      RETURN
      END

C######################################################################

      SUBROUTINE CLIM(EL, W, PCT, PCC, PCP, PCS)
C-------------------------------------------------------------------
C Purpose:
C --------
C Computes concentrations W and climate response PCT, ...,
C from emissions EL.
C
C Interface:
C ----------
C     CALL CLIM(EL, W, PCT, PCC, PCP, PCS)
C
C Input arguments:
C	EL	Emission scenario.
C
C Output arguments:
C	PCT	Temperature response (pc1 normalized to global mean)
C	...	response in cloud cover, precipitation, sea level.
C	W	CO2 concentrations.
C
C		The concentrations & climate response is computed for
C		time steps numbered 0, ... , NCLIM. In original SIAM,
C		NCLIM is NTAU+1.
C
C Externals:
C ----------
C	CONVCARC	Compute concentrations from emissions.
C	CONVCLIC	Compute temperatures from concentrations.
C
C Author:
C -------
C G.Hooss, MPI Me HH, Dec 1996.
C
C Changes:
C --------
C Renamed to CLIM as is also the nonlinear climate model in another
C source file. Choice of climate model is done through choice
C of source file, no longer through changes in the source itself.
C-------------------------------------------------------------------

      IMPLICIT NONE

#include "spartime.h"
#include "scmatbox.h"
#include "sccarbon.h"

      INTEGER J

      REAL EL(0:NTAU), W(0:NTAU),
     .     PCT(0:NTAU), PCC(0:NTAU), PCP(0:NTAU), PCS(0:NTAU)

C----------------------------------------------------------------------
C Accumulate emissions.
C----------------------------------------------------------------------

      ELACC(0)=0
      DO J=1,NTAU
        ELACC(J) = ELACC(J-1) + 0.5 * DT * (EL(J-1) + EL(J))
      END DO

C----------------------------------------------------------------------
C Call linear ocean CO2 module and linear global-mean T2m module.
C----------------------------------------------------------------------

      CALL CONVCARC(EL, W)
      CALL CONVCLIC(W, PCT, PCC, PCP, PCS)

      RETURN
      END

C######################################################################
