C######################################################################
C NICAOS
C======================================================================
C
C Name:
C -----
C
C Nonlinear Impulse-response representation of the Coupled
C Atmospher-Ocean climate System
C
C Climate-change-module-alone version for concentration scenarios.
C
C
C Purpose:
C --------
C Translate scenario of atmospheric CO2 concentration
C into corresponding greenhouse-effect climate change.
C
C
C I/O:
C ----
C
C In:
C      climate module paramters
C      concentration scenario
C        (time series of atmospheric CO2 concentrations)
C
C Out: global-mean T2m,
C      cloud cover (clo),
C      precipitation (pre) and
C      sea level change (sea) time series.
C
C
C Compilation:
C ------------
C
C NLIRFMOD.F can be compiled / linked
C  with NICAOS.F instead of NICCS.F or LICCS.F.
C NLIRFMOD calls CLIM and gets the one linked by the compiler.
C
C
C Method:
C -------
C
C The NLIRFMOD main program gets a concentration scenario to read
C  (instead of emissions). The concentrations are then passed to
C the first argument of CLIM (just as the emissions in NICCS | LICCS).
C
C The NICAOS CLIM will then copy its first argument vector
C into the second one to return the concentrations
C in INITMOD's concentration array W.
C
C CLIM will subsequently set its first argument vecotr EL to zeroes
C to make clear in the INITMOD output that there were actually
C no emissions known or given.
C
C The NICAOS CLIM will then call CAO1INI and CAO1ST
C  (versions of CL1INI and CL1STP
C modified to treat only the climate part).
C 
C 
C Possible further improvement:
C -----------------------------
C 
C Although unwanted in NICAOS, the NICCS CarC computations
C  *are* performed by NICAOS CLIM,
C  however with dummy or zero values.
C 
C To save computation time for multi-scenario applications,
C  all NICCS-CarC-related code
C  could be completely removed from CAO1INI and CAO1STP.
C 
C A good deal of time is also consumed by the chemical look-up table.
C This carbon cycle initialization could be extracted from NLIRFMOD
C  and put into their own respective subroutines.
C 
C Together with the CarC box-model parameter tuning in NICOCAL.F,
C  this part could form a separate carbon-cycle initialization module
C  which could then be chosen to be linked or not.
C 
C 
C Author:
C -------
C G.Hooss MPI Meteo
C
C
C Copyright
C ---------
C (C) 2001 K.G.Hooss
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License
C as published by the Free Software Foundation; either version 2
C of the License, or (at your option) any later version.
C 
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C 
C You should have received a copy of the GNU General Public License
C along with this program; if not, write to the Free Software
C Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
C######################################################################

      SUBROUTINE CAO1INI(IT, EMISS, WATM, PCT, PCC, PCP, PCS)
C----------------------------------------------------------------------
C Purpose:
C --------
C Initialize climate state before first call to single-timestep climate
C change model. By a nonzero time index IT, the model may be started at
C any time step.
C
C Interface:
C ----------
C     CALL CAO1INI(IT, EMISS, WATM, PCT, PCC, PCP, PCS)
C Input argument:
C	IT	Time step index, usually zero.
C Output arguments:
C	all others, see description in CAO1STP.
C
C No externals.
C -------------
C
C Author:
C -------
C G.Hooss, MPI Meteo, 1998-11
C----------------------------------------------------------------------

      IMPLICIT NONE
      INTEGER IT, K, IB

#include "spartime.h"
#include "scparche.h"
#include "scarrche.h"
#include "scmatbox.h"
#include "scparbio.h"

#include "scparirf.h"

      REAL EMISS(0:NTAU), WATM(0:NTAU),
     .     PCT(0:NTAU), PCS(0:NTAU), PCP(0:NTAU), PCC(0:NTAU)

C----------------------------------------------------------------------
C "Local" variables: keep values until next call to CAO1STP.
C----------------------------------------------------------------------

#include "sccarbon.h"
#include "scclimvar.h"

C----------------------------------------------------------------------
C Initial values: perturbation is zero.
C----------------------------------------------------------------------

      E2HS = ETA(2) * HSINV

      ELACC(IT) = 0.0
      CTOT(IT) = 0.0

      DO IB = 1,NBX
         COC(IB,IT) = 0.0
         CBS(IB,IT) = 0.0
      END DO

      CA(IT) = CATAB(0)
      CS(IT) = CSTAB(0)

      AFN(IT)  = AFTAB(0)
      BFN(IT)  = BFTAB(0)
      DFN(IT)  = DFTAB(0)

C----------------------------------------------------------------------
C Redundant (ignored) values to unused points. For completeness only.
C----------------------------------------------------------------------

      DO IB = 1,NBX
         COCPR(IB,IT) = 0.0
         CBSPR(IB,IT) = 0.0
      END DO

      CAPR(IT) = CATAB(0)
      CSPR(IT) = CSTAB(0)

      APR(IT)  = AFTAB(0)
      BPR(IT)  = BFTAB(0)
      DPR(IT)  = DFTAB(0)

C======================================================================
C Initialize: time derivatives in IT:th point.
C----------------------------------------------------------------------

C----------------------------------------------------------------------
C Time derivative in data point.
C 1st & 2nd boxes/layers: Nonlinear equations containing cs(c1).
C----------------------------------------------------------------------

      COCDOT(1,IT) = DFN(IT) * (
     .                   EMISS(IT) - E2HS * CS(IT)
     .                   - MSYS(1,2) * COC(2,IT)
     .                   - (BHOO3+BHOO4) * BFN(IT)
     .                   + TJOO3INV * CBS(3,IT)
     .                   + TJOO4INV * CBS(4,IT)   )



      COCDOT(2,IT) = E2HS * CS(IT)
     .                    - MSYS(2,2)*COC(2,IT)
     .                    - MSYS(2,3)*COC(3,IT)

C----------------------------------------------------------------------
C 3rd & 4th boxes: model eqs in matrix form like in the linear case.
C----------------------------------------------------------------------

      DO IB = 3, NBX
         COCDOT(IB,IT) = 0.0
         DO K = 1, NBX
            COCDOT(IB,IT) = COCDOT(IB,IT)
     .                     - MSYS(IB,K) * COC(K,IT)
         END DO
      END DO

C----------------------------------------------------------------------
C Biospheric reservoirs.
C----------------------------------------------------------------------

      CBSDOT(2,IT) = AFN(IT) * COCDOT(1,IT)
      CBSDOT(3,IT) = BHOO3 * BFN(IT) - TJOO3INV * CBS(3,IT)
      CBSDOT(4,IT) = BHOO4 * BFN(IT) - TJOO4INV * CBS(4,IT)

C----------------------------------------------------------------------
C Redundant (ignored) values to unused points. For completeness only.
C----------------------------------------------------------------------

      DO IB = 1,NBX
         COCPRDOT(IB,IT) = COCDOT(IB,IT)
         CBSPRDOT(IB,IT) = CBSDOT(IB,IT)
      END DO

C----------------------------------------------------------------------
C Add unperturbed atm C to perturbation -> total atm. C load.
C----------------------------------------------------------------------

c------------------------------------------------------------
c nicaos: do not compute concentrations. Ignore carbon cycle.
c------------------------------------------------------------

c      WATM(IT) = W0 + CA(IT)

C======================================================================
C Carbon model initialization complete, begin climate initialization:
C======================================================================
C 1. Normalized forcing and its time derivative.
C----------------------------------------------------------------------

      DT2INV = 0.5/DT
      LN2INV = 1.0 / LOG(2.0)

C----------------------------------------------------------------------
C 2.a: Inverse time constants (from IRFs).
C----------------------------------------------------------------------

      RPCTT1INV = 1.0 / RPCTT1
      RPCTT2INV = 1.0 / RPCTT2

      RPCST1INV = 1.0 / RPCST1
      RPCST2INV = 1.0 / RPCST2

      RPCPT1INV = 1.0 / RPCPT1
      RPCPT2INV = 1.0 / RPCPT2

      RPCCT1INV = 1.0 / RPCCT1
      RPCCT2INV = 1.0 / RPCCT2

C----------------------------------------------------------------------
C Init "local" climate state variables for zeroth time point.
C----------------------------------------------------------------------

      FORC(IT) = LN2INV * LOG(WATM(IT)/WATM(0))

      PCT1(IT) = 0.0
      PCT2(IT) = 0.0

      PCC1(IT) = 0.0
      PCC2(IT) = 0.0

      PCP1(IT) = 0.0
      PCP2(IT) = 0.0

      PCS1(IT) = 0.0
      PCS2(IT) = 0.0

C----------------------------------------------------------------------
C End of initialization.
C----------------------------------------------------------------------

      RETURN
      END

C######################################################################

      SUBROUTINE CAO1STP(JT, EMISS, WATM, PC1T, PC1C, PC1P, PC1S)
C----------------------------------------------------------------------
C Purpose:
C --------
C Single-timestep climate model.
C Computes atmospheric carbon load WATM and climate state (TGLBM, ...)
C for anthropogenic CO2 emissions EMISS(JT) at time step index JT.
C
C !! CAUTION: Correct performance at index JT *requires* that
C	the dynamical climate state variables used by this
C	routine must be known for the previous time step.
C	The dynamical variables are therefore combined into COMMON blocks
C	(although they are not actually supposed to be used outside CAO1STP,
C	except by the initialization routine CAO1INI) so their values
C	will be saved until the next call to CAO1STP.
C	So before any call to CAO1STP for time index JT, the dynamical
C	variables must be assigned values for JT-1 by one of the following:
C   (a) by a call to the inititalization routine CAO1INI for time step JT-1
C	(at the beginning of the time integration, usually at JT=1), or,
C   (b) by a previous call to CAO1STP for time step JT-1 .
C
C Interface:
C ----------
C      CALL CAO1STP(JT, EMISS, WATM, PC1T, PC1C, PC1P, PC1S)
C
C Input arguments:
C	JT	int		time step index
C	EMISS	real(0:ntau)	anthropogenic carbon emissions (GtC/y)
C Output arguments:
C	WATM	real(0:ntau)	atmospheric carbon load (GtC)
C	PC1T	real(0:ntau)	1st PC of 2m temperature change
C	PC1C	real(0:ntau)	1st PC of cloud coverage change
C	PC1P	real(0:ntau)	1st PC of precipitation change
C	PC1S	real(0:ntau)	1st PC of sea level rise.
C
C Technical note:
C ---------------
C All internal dynamical variables describing the time-dependent
C climate state are vectors/matrices with the time index JT numbering
C their elements (0:NTAU).
C They are combined into a COMMON block in order to keep their values
C from call to call.
C
C Method:
C -------
C (a) Carbon cycle
C
C Version 5: The linear IRF convolution is replaced by a linear
C differential analogue with 4 reservoirs. This model is based on
C the assumption that the sea surface layer is in CO2 equilibrium
C with the atmosphere; so there is one combined box for both subsystems.
C This allows to fit the response to an IRF that is appropriately
C reduced by its shortest time scale, and to integrate the model by
C time steps long compared to the diffusion of CO2 through the surface.
C The (Newton-relaxation type) equations are integrated by a simple
C predictor-corrector scheme that provides high accuracy along with
C numerical stability up to time steps slightly longer than the
C shortest time scale explicitly integrated.
C
C In a second step, the partition of the carbon perturbation
C in the combi box is, at each time step, computed by a nonlinear
C chemistry module.
C
C Version 11: Inclusion of a land vegetation model:
C The model is based on the Joos CO2-fertilization model with the
C global NPP forced logarithmically by atmospheric CO2, and
C the respirative C flow back into the atmosphere described by
C a convolution of the enhanced NPP with a linear IRF of the
C organic decay. The IRF is the sum of 4 decaying exponentials.
C
C The modified version implemented here is developed in 4 steps:
C 1. Find an equivalent IRF model for the additionally sequestered
C    carbon (instead of its return flow).
C 2. Reformulate the equivalent IRF model into differential
C    equations (instead of convolution integral). There
C    will be one carbon reservoir corresponding to each of the
C    4 time scales of the IRF, each with one uncoupled differential
C    equation containing a decay term and a forcing term (NPP).
C 3. Two of the decay time scales are shorter than the numerical
C    time step of the SIAM model (dt = 5 y). Replace the
C    corresponding reservoirs (CB1, CB2) and their decay
C    equations by a correction reservoir the C content of which
C    is just proportional to the NPP flow. This represents the
C    carbon that is sequestered *and* returned to the atmosphere
C    during one single time step.
C 4. The model is to be forced by the atmospheric carbon CA which
C    means that its three reservoirs are to be coupled into the
C    equation for the combined atmosphere-sea-surface system C1.
C    The c1dot equation then contains several nonlinear functions
C    of c1: cs, B, D. At each time step, their respective values
C    are read from tables generated by INITMOD (ca & cs by call
C    to PARTIT).
C
C Units are GtC, resp. GtC/y.
C
C Technical realization: Matrices COC & CBS have one column
C per time step, containing in the rows the respective values of
C    COC: c1,         c2,        c3,  c4     (ocean carbon),
C    CBS: cbc direct, cbc cumul, cb3, cb4    (biosphere carbon).
C Row vectors:
C	CA	atmospheric carbon
C	CS	sea surface layer carbon
C	AFN, BFN, DFN
C		auxiliaries (nonlinear functions of c1).
C All of these have their predictor counterparts (-PR).
C
C The model equations for c1, ..., c4, cbc cumul, cb3, cb4
C (anthropogenic perturbations, not total carbon) are integrated
C by a simple predictor-corrector scheme.
C
C
C
C (b) Climate response:
C
C The response of the following variables to the enhanced GHG
C concentrations (WATM) is computed by a differential analogue to
C the pure impulse-response formulation with logarithmic radiative
C forcing:
C	T2m (global mean: TBLM,	 PC1: PC1T)
C	Cloudiness		(PC1: PC1C)
C	Precipitation		(PC1: PC1P)
C	Sea level rise		(PC1: PC1S)
C
C The integration is performed by a simple predictor-corrector scheme
C that is formulated in compact one-step equations.
C
C Units are GtC, degrees Celsius, cloud cover fraction (0...1),
C mm/day (precipitation), and metres (sea level).
C Correct physical values of climate change at location x and time t
C is the product of the EOF value (dimensionless) at location x
C times the PC value provided by this model (with the unit listed above)
C at time t.
C
C Version 13 (1998-11): Climate change module combined with carbon
C cycle into a single-timestep module.
C
C No externals.
C -------------
C
C References:
C -----------
C K.Hasselmann: Differential form & modified numerics of the Joos
C nonlinear global carbon cycle model (personal communication, 1997).
C
C Author:
C -------
C Georg Hooss, MPI Me HH, 1997-8, 1998-3, 1998-11, 2001-06
C----------------------------------------------------------------------

      IMPLICIT NONE

#include "spartime.h"
#include "scparche.h"
#include "scarrche.h"
#include "scparirf.h"
#include "scclimvar.h"
#include "scmatbox.h"
#include "scparbio.h"

      INTEGER JT

      REAL EMISS(0:NTAU), WATM(0:NTAU)
      REAL PC1T(0:NTAU), PC1C(0:NTAU), PC1P(0:NTAU), PC1S(0:NTAU)

      INTEGER IB, K

C----------------------------------------------------------------------
C "Local" variables: keep values until next call.
C----------------------------------------------------------------------

#include "sccarbon.h"

C======================================================================
C Predictor contents: NBX ocean boxes, 3 biosphere reservoirs.
C----------------------------------------------------------------------

      DO IB = 1, NBX
         COCPR(IB,JT) = COC(IB,JT-1) + COCDOT(IB,JT-1) * DT
      END DO
      DO IB = 2,4
         CBSPR(IB,JT) = CBS(IB,JT-1) + CBSDOT(IB,JT-1) * DT
      END DO

C----------------------------------------------------------------------
C Predictor contents: partition into atmosphere & surface layer
C and auxiliary functions. Plus direct biosph corr term.
C----------------------------------------------------------------------

      K = INT(DELCINV * COCPR(1,JT))

      RP(JT)    =  DELCINV * (COCPR(1,JT) - K*DELC1)
      CAPR(JT)  = CATAB(K) + RP(JT) * (CATAB(K+1)-CATAB(K))
      CSPR(JT)  = CSTAB(K) + RP(JT) * (CSTAB(K+1)-CSTAB(K))
      APR(JT)   = AFTAB(k) + RP(JT) * (AFTAB(K+1)-AFTAB(K))
      BPR(JT)   = BFTAB(k) + RP(JT) * (BFTAB(K+1)-BFTAB(K))
      DPR(JT)   = DFTAB(k) + RP(JT) * (DFTAB(K+1)-DFTAB(K))

      CBSPR(1,JT) = FCORR * BPR(JT)

C======================================================================
C Predictor time derivative.
C 1st & 2nd line: Nonlinear equations containing cs(c1).
C----------------------------------------------------------------------

      COCPRDOT(1,JT) = DPR(JT) * (
     .                 EMISS(JT) - E2HS * CSPR(JT)
     .                 - MSYS(1,2) * COCPR(2,JT)
     .                 - (BHOO3+BHOO4) * BPR(JT)
     .                 + TJOO3INV * CBSPR(3,JT)
     .                 + TJOO4INV * CBSPR(4,JT)   )



      COCPRDOT(2,JT) = E2HS * CSPR(JT)
     .                 - MSYS(2,2)*COCPR(2,JT)
     .                 - MSYS(2,3)*COCPR(3,JT)

C----------------------------------------------------------------------
C 3rd & 4th boxes: model eqs in matrix form like in the linear case.
C----------------------------------------------------------------------

      DO IB = 3, NBX
         COCPRDOT(IB,JT) = 0.0
         DO K = 1, NBX
            COCPRDOT(IB,JT) = COCPRDOT(IB,JT)
     .                        - MSYS(IB,K) * COCPR(K,JT)
         END DO
      END DO

C----------------------------------------------------------------------
C Biospheric reservoirs.
C----------------------------------------------------------------------

      CBSPRDOT(2,JT) = APR(JT) * COCPRDOT(1,JT)
      CBSPRDOT(3,JT) = BHOO3 * BPR(JT) - TJOO3INV * CBSPR(3,JT)
      CBSPRDOT(4,JT) = BHOO4 * BPR(JT) - TJOO4INV * CBSPR(4,JT)

C======================================================================
C New point box contents: NBX ocean boxes, 3 biosphere reservoirs.
C----------------------------------------------------------------------

      DO IB = 1, NBX
         COC(IB,JT) = COC(IB,JT-1) + 0.5 * DT *
     .                (COCDOT(IB,JT-1) + COCPRDOT(IB,JT))
      END DO
      DO IB = 2,4
         CBS(IB,JT) = CBS(IB,JT-1) + 0.5 * DT *
     .                (CBSDOT(IB,JT-1) + CBSPRDOT(IB,JT))
      END DO

C----------------------------------------------------------------------
C New point: partition into atmosphere & surface layer
C and auxiliary functions. Plus direct biosph corr term.
C----------------------------------------------------------------------

      K = INT(DELCINV * COC(1,JT))

      RC(JT)    =  DELCINV * (COC(1,JT) - K*DELC1)
      CA(JT)    = CATAB(K) + RC(JT) * (CATAB(K+1)-CATAB(K))
      CS(JT)    = CSTAB(K) + RC(JT) * (CSTAB(K+1)-CSTAB(K))
      AFN(JT)   = AFTAB(k) + RC(JT) * (AFTAB(K+1)-AFTAB(K))
      BFN(JT)   = BFTAB(k) + RC(JT) * (BFTAB(K+1)-BFTAB(K))
      DFN(JT)   = DFTAB(k) + RC(JT) * (DFTAB(K+1)-DFTAB(K))

      CBS(1,JT) = FCORR * BFN(JT)

C======================================================================
C Time derivative in new point.
C 1st & 2nd boxes/layers: Nonlinear equations containing cs(c1).
C----------------------------------------------------------------------

      COCDOT(1,JT) = DFN(JT) * (
     .                 EMISS(JT) - E2HS * CS(JT)
     .                 - MSYS(1,2) * COC(2,JT)
     .                 - (BHOO3+BHOO4) * BFN(JT)
     .                 + TJOO3INV * CBS(3,JT)
     .                 + TJOO4INV * CBS(4,JT)   )

      COCDOT(2,JT) = E2HS * CS(JT)
     .               - MSYS(2,2)*COC(2,JT)
     .               - MSYS(2,3)*COC(3,JT)

C----------------------------------------------------------------------
C 3rd & 4th boxes: model eqs in matrix form like in the linear case.
C----------------------------------------------------------------------

      DO IB = 3, NBX
         COCDOT(IB,JT) = 0.0
         DO K = 1, NBX
            COCDOT(IB,JT) = COCDOT(IB,JT)
     .                      - MSYS(IB,K) * COC(K,JT)
         END DO
      END DO

C----------------------------------------------------------------------
C Biospheric reservoirs.
C----------------------------------------------------------------------

      CBSDOT(2,JT) = AFN(JT) * COCDOT(1,JT)
      CBSDOT(3,JT) = BHOO3 * BFN(JT) - TJOO3INV * CBS(3,JT)
      CBSDOT(4,JT) = BHOO4 * BFN(JT) - TJOO4INV * CBS(4,JT)

C======================================================================
C Accumulate anthropogenic carbon for checking purposes.
C----------------------------------------------------------------------

      ELACC(JT) = ELACC(JT-1) + 0.5 * DT * (EMISS(JT-1) + EMISS(JT))
      CTOT(JT) = COC(1,JT) + COC(2,JT) + COC(3,JT) + COC(4,JT) +
     .           CBS(1,JT) + CBS(3,JT) + CBS(4,JT)

C======================================================================
C Add unperturbed atm C to perturbation -> total atm. C load.
C----------------------------------------------------------------------

c------------------------------------------------------------
c nicaos: do not compute concentrations. Ignore carbon cycle.
c------------------------------------------------------------

c      WATM(JT) = W0 + CA(JT)

C======================================================================
C Carbon model time step complete.
C Begin climate response time step.
C======================================================================

C------------------------------------------------------------------
C 1. Normalized forcing.
C------------------------------------------------------------------

      FORC(JT) = LN2INV * LOG(WATM(JT)/W0)

C----------------------------------------------------------------------
C 2.b: Predictor-corrector (rewritten to one-step) for 7 variables:
C      3 mean temperature reservoirs,
C      2 pc1 reservoirs for:
C            temperature, sea level, precipitation, cloudiness.
C------------------------------------------------------------------
C left over for curiosity from old SIAM irf model:
C Mean temperature: 1st time constant is shorter than time step.
C Approximate by immediate adjustment (instead of relaxation term).
C----------------------------------------------------------------------
c
c      T1(JT) = T1(JT-1) + RTA1*(FORC(JT)-FORC(JT-1))
c      T2(JT) = T2(JT-1) + DT*0.5*RTT2INV
c     .           * (RTA2*(FORC(JT-1)+FORC(JT))
c     .              - (2*T2(JT-1) + DT*RTT2INV
c     .                 * (RTA2*FORC(JT-1)-T2(JT-1))))
c      T3(JT) = T3(JT-1) + DT*0.5*RTT3INV
c     .           * (RTA3*(FORC(JT-1)+FORC(JT))
c     .              - (2*T3(JT-1) + DT*RTT3INV
c     .                 * (RTA3*FORC(JT-1)-T3(JT-1))))

C----------------------------------------------------------------------
C Temperature PC1.
C----------------------------------------------------------------------
      PCT1(JT) = PCT1(JT-1) + DT*0.5*RPCTT1INV
     .           * (RPCTA1*(FORC(JT-1)+FORC(JT))
     .              - (2*PCT1(JT-1) + DT*RPCTT1INV
     .                 * (RPCTA1*FORC(JT-1)-PCT1(JT-1))))
      PCT2(JT) = PCT2(JT-1) + DT*0.5*RPCTT2INV
     .           * (RPCTA2*(FORC(JT-1)+FORC(JT))
     .              - (2*PCT2(JT-1) + DT*RPCTT2INV
     .                 * (RPCTA2*FORC(JT-1)-PCT2(JT-1))))

C----------------------------------------------------------------------
C Cloudiness PC1.
C----------------------------------------------------------------------
      PCC1(JT) = PCC1(JT-1) + DT*0.5*RPCCT1INV
     .           * (RPCCA1*(FORC(JT-1)+FORC(JT))
     .              - (2*PCC1(JT-1) + DT*RPCCT1INV
     .                 * (RPCCA1*FORC(JT-1)-PCC1(JT-1))))
      PCC2(JT) = PCC2(JT-1) + DT*0.5*RPCCT2INV
     .           * (RPCCA2*(FORC(JT-1)+FORC(JT))
     .              - (2*PCC2(JT-1) + DT*RPCCT2INV
     .                 * (RPCCA2*FORC(JT-1)-PCC2(JT-1))))


C----------------------------------------------------------------------
C Precipitation PC1.
C----------------------------------------------------------------------
      PCP1(JT) = PCP1(JT-1) + DT*0.5*RPCPT1INV
     .           * (RPCPA1*(FORC(JT-1)+FORC(JT))
     .              - (2*PCP1(JT-1) + DT*RPCPT1INV
     .                 * (RPCPA1*FORC(JT-1)-PCP1(JT-1))))
      PCP2(JT) = PCP2(JT-1) + DT*0.5*RPCPT2INV
     .           * (RPCPA2*(FORC(JT-1)+FORC(JT))
     .              - (2*PCP2(JT-1) + DT*RPCPT2INV
     .                 * (RPCPA2*FORC(JT-1)-PCP2(JT-1))))

C----------------------------------------------------------------------
C Sea level PC1.
C----------------------------------------------------------------------
      PCS1(JT) = PCS1(JT-1) + DT*0.5*RPCST1INV
     .           * (RPCSA1*(FORC(JT-1)+FORC(JT))
     .              - (2*PCS1(JT-1) + DT*RPCST1INV
     .                 * (RPCSA1*FORC(JT-1)-PCS1(JT-1))))
      PCS2(JT) = PCS2(JT-1) + DT*0.5*RPCST2INV
     .           * (RPCSA2*(FORC(JT-1)+FORC(JT))
     .              - (2*PCS2(JT-1) + DT*RPCST2INV
     .                 * (RPCSA2*FORC(JT-1)-PCS2(JT-1))))

C----------------------------------------------------------------------
C Compose output variables from their respective reservoirs.
C----------------------------------------------------------------------

      PC1T(JT)   = PCT1(JT) + PCT2(JT)
      PC1C(JT)   = PCC1(JT) + PCC2(JT)
      PC1P(JT)   = PCP1(JT) + PCP2(JT)
      PC1S(JT)   = PCS1(JT) + PCS2(JT)

C======================================================================
C Climate response time step complete.
C======================================================================

      RETURN
      END

C###################################################################

      SUBROUTINE CLIM(EL, W, PCT, PCC, PCP, PCS)
C----------------------------------------------------------------------
C Purpose:
C --------
C Computes climate response (PCT, PCC, PCP, PCS)
C from concentrations W. No emissions are dealt with.
C For compatibility with LICCS and NICCS, the concentrations (ppmv)
C are here given as first argument (instead of emissions EL).
C Converted into atm C load (GtC), they are returned as second
C argument W (not computed by a carbon cycle module).
C Thereafter the first argument vector is assigned zero values
C in all components to indicate that the emissions were unknown.
C
C Interface:
C ----------
C     CALL CLIM(EL, W, PCT, PCC, PCP, PCS)
C
C Input arguments:
C	EL	Concentration scenario. Zero on exit. Emissions unknown.
C
C Output arguments:
C	W	CO2 concentrations (copy of initial EL values).
C	PCT	Temperature   (1st PC) response.
C	PCS	Sea level     (1st PC) response.
C	PCT	Precipitation (1st PC) response.
C	PCS	Cloudiness    (1st PC) response.
C
C		The climate response is computed for
C		time steps numbered 0, ... , NCLIM. In original SIAM,
C		NCLIM is NTAU+1.
C
C Externals:
C ----------
C	CAO1STP	One-step climate model. Works properly only if the
C		initialization routine:
C	CAO1INI	has been called before first call to CAO1STP.
C
C Author:
C -------
C G.Hooss, MPI Me HH, Mar 2000.
C-------------------------------------------------------------------

      IMPLICIT NONE

#include "spartime.h"
#include "scparche.h"

C----------------------------------------------------------------------
C For pulse prog output (end of routine) uncomment the following line:
c#include "sccarbon.h"
C----------------------------------------------------------------------

      INTEGER JT
      REAL EL(0:NTAU),   W(0:NTAU),
     .     PCT(0:NTAU), PCS(0:NTAU), PCP(0:NTAU), PCC(0:NTAU)
      REAL ALPHAAINV

C======================================================================
C Begin computations.
C======================================================================

C----------------------------------------------------------------------
C Assume concentrations in EL are given in ppmv.
C Convert into atm carbon load W in GtC.
C Set emissions array EL to zero (unknown emissions).
C----------------------------------------------------------------------

      ALPHAAINV = 1.0 / ALPHAA

      DO JT = 0, NTAU
         W(JT) = ALPHAAINV * EL(JT)
      END DO

      DO JT = 0, NTAU
         EL(JT) = 0
      END DO

C----------------------------------------------------------------------
C Initialize "local" variables of the one-step climate module:
C This call must be made once before the first call to CLIM13.
C----------------------------------------------------------------------

      CALL CAO1INI(0, EL, W, PCT, PCC, PCP, PCS)

C======================================================================
C Time step loop. Note that all arrays are declared 0:NTAU in the time
C dimension. This is in accord with the original SIAM time horizon.
C Since the 0th column of the output matrix will be assigned
C the initial values, the 1st time step leads to the 1st data point,
C or, generally spoken, the time index always numbers the data point
C *to which* the step is taken.
C----------------------------------------------------------------------
C
C Tell TAMC that there is no anti-flow data dependence in CC.
C
CADJ LOOP = SEQUENTIAL
C
C----------------------------------------------------------------------

      DO JT = 1, NTAU
         CALL CAO1STP(JT, EL, W, PCT, PCC, PCP, PCS)
      END DO

C----------------------------------------------------------------------
C Test: output box contents. This is for the 'pulse' program.
C !! NEVER do this during the SIAM optimization !! %&#$@ !!
C----------------------------------------------------------------------
c      OPEN(19,file='s_carbon.out')
c      write(19,'(A10, 14A12)') '#     Year', 'EL',
c     .      'c1', 'c2', 'c3', 'c4',
c     .      'cbc dir', 'cbc cum', 'cb3', 'cb4',
c     .      'el accum', 'ctot', 'A', 'B', 'D'
c      write(19,'(F10.2, 14E12.4)') (jt*dt, el(jt),
c     .      coc(1,jt), coc(2,jt), coc(3,jt), coc(4,jt),
c     .      cbs(1,jt), cbs(2,jt), cbs(3,jt), cbs(4,jt),
c     .      elacc(jt), ctot(jt), afn(jt), bfn(jt), dfn(jt),
c     .      jt=0,ntau)
c      CLOSE(19)
C----------------------------------------------------------------------
      RETURN
      END

C######################################################################
