C######################################################################

      PROGRAM NLIRFMOD
C-------------------------------------------------------------------
C Name:
C -----
C NLIRFMOD is short for
C Nonlinear and Linear Impulse-Response-Function Models.
C
C Purpose:
C --------
C to be compiled together with and linked to one of the
C NICCS | LICCS | NICAOS models.
C Read emission (or concentration) scenario,
C run through the NICCS or LICCS (or NICAOS) model.
C
C Externals:
C ----------
C	PARINP
C		Read input parameters from stdin.
C	PARTIT	Seawater chemistry: partition of combilayer carbon (C1)
C		into atmosphre (CA) and sea surface layer (CS).
C	RESP	Compute impulse response functions (for check output).
C	CLIM	Compute temperature change (T) from emissions (EL).
C	(ECON3	SIAM only. Compute costs from emissions & temperatures.)
C
C History
C -------
C The three models NICCS, LICCS and NICAOS
C have been developed step by step starting in 1996
C from the original IRF-convolution climate module
C of the SIAM model (Hasselmann & al.1997).
C
C One of the purposes of this development was to confirm
C that the results of the SIAM policy optimization study
C (and indeed of any integrated assessment of climate change policy)
C are much less sensitive
C to the choice and accuracy of the climate module
C than to economic assumptions,
C especially concerning the intertemporal value assessment
C of future climate damages.
C
C The model has furthermore been used
C for further integrated assessment studies
C and for assessing climate change impacts
C on the terrestrial biosphere.
C
C References.
C -----------
C K.Hasselmann, S.Hasselmann, R.Giering, V.Ocaa, H.von Storch:
C Sensitivity study of optimal CO2 emissions paths
C  using a simplified structural integrated assessment model (SIAM).
C Climatic Change 37:345-386, 1997 
C
C K.Hasselmann:
C Intertemporal accounting of climate change -
C  Harmonizing Economic Efficiency and Climate Stewardship. 
C Climate Change 41: 303-331, 1999 
C
C F.Joos, M.Bruno, R.Fink, U.Siegenthaler, Th.F.Stocker,
C C.LeQuere, J.L.Sarmiento:
C An efficient & accurate representation of complex
C  oceanic & biospheric models of anthropogenic carbon uptake.
C Tellus 48 B, 397-417 (1996).
C
C F.Joos, M.Bruno:
C Pulse response functions are cost-efficient tools to model the link
C  between carbon emissions, atmospheric CO2, & global warming.
C Phys. & Chem. of the Earth, 1996.
C
C R.Meyer, F.Joos, G.Esser, M.Heimann, G.Hooss, G.Kohlmaier,
C  W.Sauf, R.Voss, U.Wittenberg:
C The substitution of high-resolution terrestrial biosphere models
C  and carbon sequestration in response to changing CO2 and climate,
C Global Biogeochemical cycles 13(3), 785-802 (9:1999) 
C 
C O.Deke, G.Hooss, C.Kasten, G.Klepper, K.Springer:
C Economic impact of climate change:
C  Simulation results from a coupled climate-economy model.
C Institut fuer Weltwirtschaft der Universitt Kiel,
C  Kiel Working Paper 1065, 2001. 
C 
C G.Hooss:
C  Aggregate models of climate change:
C  development and applications
C Max-Planck-Institut fr Meteorologie, Examensarbeit Nr. 83,
C Dissertation am Fachbereich Geowissenschaften
C  der Universitt Hamburg (6:2001)
C 
C G.Hooss, R.Voss, K.Hasselmann, E.Maier-Reimer, F.Joos:
C A nonlinear impulse response model of the coupled
C  carbon cycle-climate system (NICCS),
C Climate Dynamics, in press (2001) 
C 
C F.Joos, C.Prentice, S.Sitch, R.Meyer, G.Hooss,
C  G.-K.Plattner, K.Hasselmann:
C Global Warming feedbacks on terrestrial carbon uptake
C  under the IPCC emission scenarios.
C Global Biogeochemical Cycles, in press 
C 
C Th.Bruckner, G.Hooss, H.-M.Fuessel, K.Hasselmann:
C The ICLIPS climate model.
C Climate Change, submitted to special ICLIPS issue, 2001. 
C 
C Authors:
C --------
C S.Hasselmann, MPI Me HH, Apr 1995,
C U.Kilian & G.Hooss, MPI Me HH, Dec 1996.
C G.Hooss, MPI Me HH, Jan 1997, Aug 1997, Nov 2001.
C
C corresponding author:
C hooss@dkrz.de
C
C
C Copyright
C ---------
C (C) 2001 K.G.Hooss
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License
C as published by the Free Software Foundation; either version 2
C of the License, or (at your option) any later version.
C 
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C 
C You should have received a copy of the GNU General Public License
C along with this program; if not, write to the Free Software
C Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
C-------------------------------------------------------------------

      IMPLICIT NONE

#include "spartime.h"
#include "scparche.h"
#include "scparirf.h"

#include "scarrche.h"
#include "scarrirf.h"
#include "scarrmod.h"
#include "scarrref.h"
#include "scarrcost.h"
#include "scarrrr.h"

#include "scmatbox.h"
#include "scparbio.h"
#include "scsensit.h"

C----------------------------------------------------------------------
C "Local" variables in CL1STP. Needed in INITMOD for check output.
C----------------------------------------------------------------------

#include "sccarbon.h"
#include "scclimvar.h"

C----------------------------------------------------------------------

      REAL TIME(0:NTAU)
      REAL WOUT(0:NTAU)
      REAL W0INV, HPLUS
      REAL PULSE, TOFFS

      INTEGER J,K

C------------------------------------------------------------------
C 1. Init PULSE, read input variables.
C------------------------------------------------------------------

      PULSE = 0.0
      CALL PARINP

C------------------------------------------------------------------
C 2. Write input variables into an output file for check purposes.
C------------------------------------------------------------------

      OPEN(12, FILE='niccs.incheck')

      WRITE(12,*) 'Time parameters:'
      WRITE(12,'(A30,F11.3)')  ' Begin of simulation (yrs): ',
     .                            TSSP
      WRITE(12,'(A30,F11.3)')  ' End of simulation:         ',
     .                            TSSP+TAU
      WRITE(12,'(A30,F11.3)')  ' Length of simulation:      ',
     .                            TAU
      WRITE(12,'(A30,F11.3)')  ' Length of time step:       ',
     .                            DT
      WRITE(12,'(A30,I7)')     ' Number of time intervals:  ',
     .                            NTAU
      WRITE(12,*)
c      WRITE(12,'(A8,F11.3)')   ' W0   : ',W0
c      WRITE(12,*)

      WRITE(12,*) 'Carbon model layer thicknesses:'
      WRITE(12,*) (1.0/HLINV(J), J = 1,NBX), 1.0/HSINV
      WRITE(12,*) 'Diffusion coefficients:'
      WRITE(12,*)   (ETA(J), J = 1,NBX)

      WRITE(12,*)
      WRITE(12,*) 'System matrix:'
      WRITE(12,*)
      DO J = 1, NBX
         WRITE(12, '(4F16.6)') (MSYS(J,K), K=1,NBX)
      END DO
      WRITE(12,*)
      WRITE(12,*) 'Terrestrial biosphere.'
      WRITE(12,*) 'Unperturbed NPP:      ', FNPP0
      WRITE(12,*) 'Fertilization factor: ', BETANPP
      WRITE(12,*)

      WRITE(12,*) 'Near-surf tmp: Sensitivity &  IRF (RPCT) amplitudes'
      WRITE(12,'(A10,F11.3)')   ' PATTMN : ', SENSIT2M
      WRITE(12,'(A10,F11.3)')   ' RPCTA1 : ', RPCTA1
      WRITE(12,'(A10,F11.3)')   ' RPCTA2 : ', RPCTA2
      WRITE(12,*) ' ... and time constants.'
      WRITE(12,'(A10,F11.3)')   ' RPCTT1 : ', RPCTT1
      WRITE(12,'(A10,F11.3)')   ' RPCTT2 : ', RPCTT2
      WRITE(12,*)
      WRITE(12,*) 'Cloud cover:   Sensitivity & IRF (RPCS) ampl.'
      WRITE(12,'(A10,F11.3)')   ' PATCMN : ', SENSICLO
      WRITE(12,'(A10,F11.3)')   ' RPCCA1 : ', RPCCA1
      WRITE(12,'(A10,F11.3)')   ' RPCCA2 : ', RPCCA2
      WRITE(12,*) ' ... and time constants.'
      WRITE(12,'(A10,F11.3)')   ' RPCCT1 : ', RPCCT1
      WRITE(12,'(A10,F11.3)')   ' RPCCT2 : ', RPCCT2
      WRITE(12,*)
      WRITE(12,*) 'Precipitation: Sensitivity & IRF (RPCT) ampl.'
      WRITE(12,'(A10,F11.3)')   ' PATPMN : ', SENSIPRE
      WRITE(12,'(A10,F11.3)')   ' RPCPA1 : ', RPCPA1
      WRITE(12,'(A10,F11.3)')   ' RPCPA2 : ', RPCPA2
      WRITE(12,*) ' ... and time constants.'
      WRITE(12,'(A10,F11.3)')   ' RPCPT1 : ', RPCPT1
      WRITE(12,'(A10,F11.3)')   ' RPCPT2 : ', RPCPT2
      WRITE(12,*)
      WRITE(12,*) 'Sea level:     Sensitivity & IRF (RPCS) ampl.'
      WRITE(12,'(A10,F11.3)')   ' PATSMN : ', SENSISEA
      WRITE(12,'(A10,F11.3)')   ' RPCSA1 : ', RPCSA1
      WRITE(12,'(A10,F11.3)')   ' RPCSA2 : ', RPCSA2
      WRITE(12,*) ' ... and time constants.'
      WRITE(12,'(A10,F11.3)')   ' RPCST1 : ', RPCST1
      WRITE(12,'(A10,F11.3)')   ' RPCST2 : ', RPCST2
      WRITE(12,*)

      CLOSE(12)

C----------------------------------------------------------------------
C 3. Compute lookup table of combined-layer carbon partitions.
C----------------------------------------------------------------------
C !! Note: SIAM eventually tries strong negative emissions during the
C optimization procedure; these can lead to considerable negative
C perturbations in the carbon budget, and thus to negative table
C indices. To prevent a segmentation fault crash, the tables have
C to be extended far into the negative regime.
C  However, since there are no realistic states connected with these
C negative carbon contents, the dummy values of the corresponding
C table entries do not really matter, as long as they do not lead to
C numerical instabilities or optimization bias away from the optimum.
C  They are, therefore, assigned the respective values for zero
C perturbation, just in order to keep *any* value. This will not
C lead to severe errors even in the (physically possible) case of small
C negative perturbations (nevertheless, these would be treated
C erroneously).
C----------------------------------------------------------------------

      CALL PARTIT(SIGC0, 1.0/HSINV, 0.0, HH0,
     .     HHTAB(0), PCTAB(0), CATAB(0), CSTAB(0))

      DO J = 1, NC1
         HPLUS = HHTAB(J-1)
         CALL PARTIT(SIGC0, 1.0/HSINV, J*DELC1, HPLUS,
     .        HHTAB(J), PCTAB(J), CATAB(J), CSTAB(J))
      END DO

      DO J = -1, -NEGC1, -1
         HPLUS = HHTAB(J+1)
         CALL PARTIT(SIGC0, 1.0/HSINV, J*DELC1, HPLUS,
     .        HHTAB(J), PCTAB(J), CATAB(J), CSTAB(J))
      END DO

      DO J = -10*NC1, -NEGC1-1
         HHTAB(J) = HHTAB(-NEGC1)
         PCTAB(J) = PCTAB(-NEGC1)
         CATAB(J) = CATAB(-NEGC1)
         CSTAB(J) = CSTAB(-NEGC1)
      END DO

C----------------------------------------------------------------------
C 3.1 Compute lookup table of dCA/dC1
C (num.deriv, 2nd order extrapolation into end point).
C----------------------------------------------------------------------
C No need for 2nd order extrapolation into zero perturbation point
C since the table is prolonged into negative domain:
c      DCADC1(-NC1) = D2C1INV * (4*CATAB(1-NC1) -   CATAB(2-NC1)
c     .                                         - 3*CATAB(-NC1))
C----------------------------------------------------------------------

      DO J = -10*NC1, NC1-1
         DCADC1(J) = D2C1INV * (CATAB(J+1) - CATAB(J-1))
      END DO
      DCADC1(NC1)  = D2C1INV * (3*CATAB(NC1)   +   CATAB(NC1-2)
     .                                         - 4*CATAB(NC1-1))

C----------------------------------------------------------------------
C 3.2 Compute lookup table of nonlinear functions A(C1), B(C1), D(C1).
C     BFTAB is delta f_npp as function of C1.
C----------------------------------------------------------------------

      W0INV = 1.0 / W0
      DO J = -NEGC1, NC1
         AFTAB(J) = FCORR * NPPFAC * DCADC1(J) / (W0 + CATAB(J))
         BFTAB(J) = FNPP0 * BETANPP * LOG(W0INV * (w0 + CATAB(J)))
         DFTAB(J) = 1.0 / (1.0 + AFTAB(J))
      END DO

      DO J = -10*NC1, -NEGC1-1
         AFTAB(J) = AFTAB(-NEGC1)
         BFTAB(J) = BFTAB(-NEGC1)
         DFTAB(J) = DFTAB(-NEGC1)
      END DO

C----------------------------------------------------------------------
C 3.3 Output lookup tables into file (for checking purposes).
C----------------------------------------------------------------------
c
c      OPEN(16,file='s_ctable.out')
c      WRITE(16,'(A8,9A16)') '#       ', 'C1',
c     .                      'HPLUS', 'PCO2', 'CA', 'CS', 'dCA/dC1',
c     .                      'A(C1)', 'B(C1)', 'D(C1)'
c
c      WRITE(16,'(I8,9E16.6)') (J, J*DELC1,
c     .     HHTAB(J), PCTAB(J), CATAB(J), CSTAB(J),
c     .     DCADC1(J), AFTAB(J), BFTAB(J), DFTAB(J), J=-NEGC1,NC1)
c      CLOSE(16)
c
C======================================================================
C 4.0 Read emission scenario.
C----------------------------------------------------------------------

      DO J = 0,NTAU
        READ(*,*) TIME(J), EA(J)
      END DO

C----------------------------------------------------------------------
C 4.1 If scenario starts with delta-impulse
C     (first point read is numbered zero):
C     nonzero 0:th value is assumed to be a factor
C     to be multiplied by preindustrial atm carbon load W0 and
C     to be
C     To convert it into an emission value, the amount is already
C     distributed over the first half time step.
C----------------------------------------------------------------------

      IF(EA(1).EQ.0.0) THEN
        IF((EA(0).GT.0.00001).OR.(EA(0).LT.-0.00001)) THEN
          EA(0) = EA(0) * W0
          PULSE = EA(0) * 0.5 * DT
        END IF
      END IF

C----------------------------------------------------------------------
C 4.2 Initialize inverse reference emissions in order to save cpu time
C   by avoiding redundant divisions.
C   Avoid division through zero by very small additional constant.
C----------------------------------------------------------------------

      DO J = 0, NTAU
        EAINV(J) = 1.0 / (EA(J) + 1.e-15)
      END DO

C======================================================================
C 5. First guess of emissions (EL) to be optimized: equal to reference BAU.
C----------------------------------------------------------------------

      DO J = 0,NTAU
        EL(J) = EA(J)
      END DO

C----------------------------------------------------------------------
C 8. Compute response functions (for check output only).
C----------------------------------------------------------------------

c      CALL RESP12(RW, RT, RPCT, RPCS, RPCP, RPCC)
      CALL RESP(RW, RPCT, RPCC, RPCP, RPCS)

C----------------------------------------------------------------------
C 9 Compute reference scenario concentrations & temperatures.
C   Choose climate module by linking with proper model source file:
C   siconvclim02.F for the linear convolution model,
C   sinboxclim00.F for the nonlinear box analogue.
C----------------------------------------------------------------------

c      CALL CLIM(EL, W, T, PCT, PCC, PCP, PCS)
      CALL CLIM(EL, W, PCT, PCC, PCP, PCS)

C----------------------------------------------------------------------
C 9 Compute reference scenario concentrations & temperatures.
C   Original linear IRF convolution climate module.
C   Emissions are not accumulated by this module, so we do it here.
C----------------------------------------------------------------------
c
c      ELACC(0)=0
c      DO J=1,NTAU
c        ELACC(J) = ELACC(J-1) + 0.5 * DT * (EL(J-1) + EL(J))
c      END DO
c
c      CALL CONVCLIM01(EL, W, T, PCT, PCC, PCP, PCS)
c
C======================================================================
C 12. Output of initial emissions, concentrations, temperatures.
C     Factor ALPHAA is for conversion GtC -> ppm.
C======================================================================
cC     If the model is applied to a delta-impulse, output normalized
cC     perturbation (WOUT) without time offset; else concentration in ppm.
cC----------------------------------------------------------------------
c
c      IF (PULSE.NE.0.0) THEN
c        DO J=0,NTAU
c          WOUT(J) = (W(J)-W0) / PULSE
c        END DO
c        TOFFS = 0.0
c      ELSE
c        DO J=0,NTAU
c          WOUT(J) =  ALPHAA * W(J)
c        END DO
c        TOFFS = TSSP
c      END IF
c
C======================================================================
      DO J=0,NTAU
        WOUT(J) =  ALPHAA * W(J)
      END DO
      TOFFS = TSSP

      OPEN(14,file='niccs.out')
      WRITE(14,'(A8,7A14)') '#   year', 'E /(GtC/y)', 'W /ppm',
     .        'T2M /degC', 'CLO /%', 'PRE /(mm/d)', 'SEA /m',
     .        'EACCUM/GtC'
      WRITE(14,'(F8.2,7E14.6)') (J*DT+TOFFS, EL(J), WOUT(J),
     .        PCT(J), PCC(J), PCP(J), PCS(J), ELACC(J),
     .        J=0,NTAU)
      CLOSE(14)

C----------------------------------------------------------------------
C 12.a Output complete system state at all times.
C----------------------------------------------------------------------
c
c      OPEN(21,file='s_refsys1.out')
c      WRITE(21,'(A8,10A14)') '#   year',
c     .  'c1 / GtC', 'c2 / GtC', 'c3 / GtC', 'c4 / GtC',
c     .  'ca / GtC', 'cs / GtC',
c     .  'cB1 / GtC', 'cB2 / GtC', 'cB3 / GtC', 'cB4 / GtC'
c
c      WRITE(21,'(F8.2,10E14.6)') (J*DT+TSSP,
c     .   COC(1,J), COC(2,J), COC(3,J), COC(4,J),
c     .   CA(J), CS(J),
c     .   CBS(1,J), CBS(2,J), CBS(3,J), CBS(4,J), J=0,NTAU)
c       CLOSE(21)
c
c      OPEN(22,file='s_refsys2.out')
c      WRITE(22,'(A8,9A14)') '#   year',
c     .  'E / (GtC/y)', 'pCO2 / ppm', 'ctot / GtC', 'Ecum / GtC',
c     .  'FRAD', 'T1 / degC', 'T2 / degC', 'T3 / degC', 'T / degC'
c 
c      WRITE(22,'(F8.2,9E14.6)') (J*DT+TSSP,
c     .   EL(J), ALPHAA*W(J), CTOT(J), ELACC(J),
c     .   FORC(J), T1(J), T2(J), T3(J), T(J), J=0,NTAU)
c      CLOSE(22)
c
C----------------------------------------------------------------------
C 13. Output response functions.
C----------------------------------------------------------------------
c
c      OPEN(15, FILE='s_irf.out')
c      WRITE(15,'(A8, 6A12)') '#   year', 'RW', 'RT',
c      WRITE(15,'(A8, 5A12)') '#   year', 'RW',
c     .                       'RPCT', 'RPCS', 'RPCC', 'RPCP'
c      WRITE(15,'(F8.2,5E12.4)') (J*DT, RW(J),
c     .                           RPCT(J), RPCS(J),
c     .                           RPCC(J), RPCP(J), J=0,NTAU)
c      CLOSE(15)
c
C----------------------------------------------------------------------

      END

C######################################################################

      SUBROUTINE SYSMAT(NB, HLINV, ETA, MSYS)
C----------------------------------------------------------------------
C Purpose:
C --------
C Build system matrix from layer thicknesses & diffusion coefficients
C for the CO2 box model.
C
C Interface:
C ----------
C     CALL SYSMAT(NB, HLINV, ETA, MSYS)
C
C Input arguments:
C	NB	Dimension of the box model (number of boxes).
C	HLINV	Vec(NB) containing inverse layer thicknesses.
C	ETA	Vec(NB)	containing the diffusion coeffs. Note that
C		 the 1:st one is not used by the model.
C
C Output argument:
C	MSYS	Mat(NB,NB) describing the model.
C
C No Externals.
C -------------
C
C References:
C -----------
C K.Hasselmann: Differential form & modified numerics of the Joos
C nonlinear global carbon cycle model (personal communication, 1997).
C
C Author:
C -------
C G.Hooss, MPI Met, Jul 1997
C----------------------------------------------------------------------

      IMPLICIT NONE
      INTEGER NB, J, K
      REAL ETA(NB), HLINV(NB), MSYS(NB,NB)

      DO J = 1,NB
         DO K = 1,NB
            MSYS(J,K) = 0.0
         END DO
      END DO


      MSYS(1,1) =   ETA(2) * HLINV(1)
      MSYS(1,2) = - ETA(2) * HLINV(2)

      DO J = 2, NB-1
         MSYS(J,J-1) = - ETA(J)           * HLINV(J-1)
         MSYS(J,J)   =  (ETA(J)+ETA(J+1)) * HLINV(J)
         MSYS(J,J+1) = - ETA(J+1)         * HLINV(J+1)
      END DO

      MSYS(NB,NB-1)  = -ETA(NB) * HLINV(NB-1)
      MSYS(NB,NB)    =  ETA(NB) * HLINV(NB)

      RETURN
      END

C######################################################################

      SUBROUTINE PARINP
C----------------------------------------------------------------------
C Purpose:
C --------
C Read input parameters from standard input.
C Input data are used to initialize 3 common blocks that are contained
C in the files scparirf.h (impulse-response-related parameters),
C scparcli.h (climate-related parameters), and scpareco.h (economic
C parameters).
C
C Interface:
C ----------
C     CALL PARINP16
C No arguments.
C 
C External:
C ---------
C	SYSMAT	Make system matrix for CO2 box diffusion model.
C
C Author:
C -------
C G.Hooss, MPI Met, Dec 1996.
C
C Version 5:
C ----------
C Read box model parameters instead of RW parameters.
C G.Hooss, MPI Met, Jul 1997.
C
C Version 6:
C ----------
C Reordered, augmented by biosphere parameters.
C G.Hooss, MPI Met, Oct 1999..
C
C Version 16:
C ----------
C Climate change EOF pattern means go with the RPC parameters.
C G.Hooss, MPI Met, Oct 1999.
C
C Version 1602:
C ----------
C Climate change sensitivities go with 1-normalized RPC parameters.
C G.Hooss, MPI Met, Jul 2000.
C----------------------------------------------------------------------

      IMPLICIT NONE

#include "scmatbox.h"
#include "scparbio.h"
#include "scparche.h"
#include "scsensit.h"
#include "scparirf.h"

      INTEGER J
      REAL PCO2_0, XIDUM

C----------------------------------------------------------------------
C IRF parameters: Amplitudes & time constants to R_w.
C----------------------------------------------------------------------

      READ(*,*) RWA0, RWA1, RWA2, RWA3, RWA4
      READ(*,*)       RWT1, RWT2, RWT3, RWT4

C----------------------------------------------------------------------
C Chemistry parameters.
C----------------------------------------------------------------------

      READ(*,*) PCO2_0, ALK, BTS, AOC      
      READ(*,*) AK1, AK2, AKB, AKW
      READ(*,*) ALPHAA, ALPHAS, SIGC0, XIDUM

      W0 = PCO2_0 / ALPHAA

C----------------------------------------------------------------------
C Land biosphere parameters.
C----------------------------------------------------------------------

      READ(*,*) FNPP0, BETANPP

C----------------------------------------------------------------------
C R_decay.
C----------------------------------------------------------------------

      READ(*,*) BJOO1, BJOO2, BJOO3, BJOO4
      READ(*,*) TJOO1, TJOO2, TJOO3, TJOO4

C----------------------------------------------------------------------
C Climate response IRFs.
C----------------------------------------------------------------------

      READ(*,*)       SENSIT2M
      READ(*,*)       RPCTA1, RPCTA2
      READ(*,*)       RPCTT1, RPCTT2
      RPCTA1 = RPCTA1 * SENSIT2M
      RPCTA2 = RPCTA2 * SENSIT2M

      READ(*,*)       SENSICLO
      READ(*,*)       RPCCA1, RPCCA2
      READ(*,*)       RPCCT1, RPCCT2
      RPCCA1 = RPCCA1 * SENSICLO
      RPCCA2 = RPCCA2 * SENSICLO

      READ(*,*)       SENSIPRE
      READ(*,*)       RPCPA1, RPCPA2
      READ(*,*)       RPCPT1, RPCPT2
      RPCPA1 = RPCPA1 * SENSIPRE
      RPCPA2 = RPCPA2 * SENSIPRE

      READ(*,*)       SENSISEA
      READ(*,*)       RPCSA1, RPCSA2
      READ(*,*)       RPCST1, RPCST2
      RPCSA1 = RPCSA1 * SENSISEA
      RPCSA2 = RPCSA2 * SENSISEA

C----------------------------------------------------------------------
C  Layer thicknesses & diffusion coefficients from tune.
C----------------------------------------------------------------------

      READ(*,*) (HLINV(J), J = 1,NBX), HSINV
      READ(*,*)   (ETA(J), J = 1,NBX)

C----------------------------------------------------------------------
C Input finished.
C----------------------------------------------------------------------

C----------------------------------------------------------------------
C Biospheric pool IRF.
C----------------------------------------------------------------------

      BHOO1 = BJOO1 * TJOO1
      BHOO2 = BJOO2 * TJOO2
      BHOO3 = BJOO3 * TJOO3
      BHOO4 = BJOO4 * TJOO4

      TJOO1INV =  1.0 / TJOO1
      TJOO2INV =  1.0 / TJOO2
      TJOO3INV =  1.0 / TJOO3
      TJOO4INV =  1.0 / TJOO4

      NPPFAC = FNPP0 * BETANPP
      FCORR  = BHOO1 * TJOO1 + BHOO2 * TJOO2

C----------------------------------------------------------------------
C Only inverse layer thicknesses are used by the model.
C HSINV is the (inverse) surface layer thickness.
C----------------------------------------------------------------------
      DO J = 1, NBX
         HLINV(J) = 1.0 / HLINV(J)
      END DO
      HSINV = 1.0 / HSINV

      CALL SYSMAT(NBX, HLINV, ETA, MSYS)

      RETURN
      END

C######################################################################

      SUBROUTINE RESP(RW, RPCT, RPCC, RPCP, RPCS)
C----------------------------------------------------------------------
c NOTE: since all modules run in differential form,
c       It is no longer necessary to compute the IRFs.
c       and this routine is obsolete.
c       It is, however, kept as part of the code
c       in case somebody wants to plot the irf functions.
C----------------------------------------------------------------------
C Purpose:
C --------
C Compute response functions.
C
C Interface:
C ----------
C     CALL RESP(RW, RPCT, RPCC, RPCP, RPCS)
C
C Input arguments:
C	W0	Preindustrial atmospheric CO2 load (GtC).
C
C Output arguments:
C	RW	Concentration response to emissions.
C	RPCT	Temp   (1st pc) response to concentrations.
C	RPCC	Cloudiness  (1st pc) response to concentrations.
C	RPCP	Precipitation (1st pc) response to concentrations.
C	RPCS	Sea level (1st pc) response to concentrations.
C
C Externals:
C ----------
C No externals.
C
C Authors:
C --------
C S.Hasselmann, MPI Me HH, Apr 1995,
C U.Kilian & G.Hooss, MPI Me HH, Dec 1996.
C G.Hooss, MPI Me HH, Jan 1997.
C G.Hooss, MPI Me HH, Dec 1997 (RPCT, RPCS).
C G.Hooss, MPI Me HH, Apr 1998 (RPCC, RPCP).
C
C Changes:
C --------
C G.Hooss MPI Met 2000-07-20
C Do not compute old siam RT model any more.
C The PC models are now normalized to global means
C (1-normalized IRF model times sensivity).
C------------------------------------------------------------------

      IMPLICIT NONE
#include "spartime.h"
#include "scparirf.h"

      REAL RW(0:NTAU)
      REAL RPCT(0:NTAU), RPCC(0:NTAU), RPCP(0:NTAU), RPCS(0:NTAU)

C-----------------
C Local variables.
C-----------------
      REAL TI
      INTEGER NT

C-----------------------------------------
C Local variables: inverse time constants.
C-----------------------------------------
      REAL RWTI1, RWTI2, RWTI3, RWTI4
      REAL RPCTTI1, RPCTTI2
      REAL RPCCTI1, RPCCTI2
      REAL RPCPTI1, RPCPTI2
      REAL RPCSTI1, RPCSTI2

C-----------------------------------------------------------------------
C Invert time constants in order to avoid redundant division operations.
C-----------------------------------------------------------------------
      RWTI1 = 1.0/RWT1
      RWTI2 = 1.0/RWT2
      RWTI3 = 1.0/RWT3
      RWTI4 = 1.0/RWT4

      RPCTTI1 = 1.0/RPCTT1
      RPCTTI2 = 1.0/RPCTT2
      RPCCTI1 = 1.0/RPCCT1
      RPCCTI2 = 1.0/RPCCT2
      RPCPTI1 = 1.0/RPCPT1
      RPCPTI2 = 1.0/RPCPT2
      RPCSTI1 = 1.0/RPCST1
      RPCSTI2 = 1.0/RPCST2

      DO NT = 0,NTAU

        TI = NT*DT
        RW(NT) = RWA0 + RWA1*EXP(-TI*RWTI1) +
     .                  RWA2*EXP(-TI*RWTI2) +
     .                  RWA3*EXP(-TI*RWTI3) +
     .                  RWA4*EXP(-TI*RWTI4)

        RPCT(NT) = RPCTA1*(1-EXP(-RPCTTI1*TI)) +
     .             RPCTA2*(1-EXP(-RPCTTI2*TI))
        RPCC(NT) = RPCCA1*(1-EXP(-RPCCTI1*TI)) +
     .             RPCCA2*(1-EXP(-RPCCTI2*TI))
        RPCP(NT) = RPCPA1*(1-EXP(-RPCPTI1*TI)) +
     .             RPCPA2*(1-EXP(-RPCPTI2*TI))
        RPCS(NT) = RPCSA1*(1-EXP(-RPCSTI1*TI)) +
     .             RPCSA2*(1-EXP(-RPCSTI2*TI))

      END DO

      RETURN
      END

C######################################################################

      SUBROUTINE PARTIT(SIGC0, HS, C1, H0, HH, PCO2, CA, CS)
C----------------------------------------------------------------
C Purpose:
C --------
C Distribution of anthropogenic carbon content C1 in combined box
C into atmospheric (CA) and seawater-dissolved (CS) carbon.
C
C Method:
C -------
C Find correct [H+] by iterative solution (Newton method) of
C constant alkalinity.
C Compute CO2 partial pressure (ppm) from [H+], CA & CS from pCO2.
C The values for total Boron, alkalinity, and dissociation constants
C are from the literature cited below. The surface layer's preindustrial
C carbon content depends on its thickness. The unperturbed (preindustrial)
C DIC (2.089 mmol/l) from Bacastow (1981) had to be adjusted to 2.0809
C mmol/l in order to give the more recent value of the unperturbed
C pCO2 of 280 ppm.
C
C Interface:
C ----------
C      CALL PARTIT(SIGC0, HS, C1, H0, HH, PCO2, CA, CS)
C
C Input (unchanged on exit):
C	SIGC0	Preindustrial DIC concentration.
C	HS	Thickness of mixed-layer.
C	C1	Total anthropogenic carbon perturbation (in GtC)
C		in the surface layer.
C	H0	[H+] in mol/l: First guess for iteration.
C Output (initial values ignored):
C	HH	[H+] in mol/l: iteratively found solution.
C	PCO2	CO2 partial pressure (ppm).
C	CA	Carbon perturbation (GtC) in the atmosphere.
C	CS	Carbon perturbation (GtC) in the surface layer.
C
C Fixed parameters:
C -----------------
C	ALK	Constant value of the alkalinity.
C	BTS	Total borate concentration (mol/l).
C	AK1, AK2, AKB
C		Dissociation constants for (bi-)carbonate & borate (mol/l).
C	AKW	Dissociation constant of water (mol/l)^2 .
C	ALPHAA	Conversion atmosph.C (GtC) -> atm.'partial pressure'(ppm).
C	ALPHAS	Henry's law constant [H2CO3] (mol/l) -> pCO2 (ppm).
C
C Local variables:
C ----------------
C	ALPHAC	Conversion surf layer DIC (mol/l) -> layer's content (GtC).
C	C0S	Preindustrial carbon load of surface layer (GtC).
C
C
C No externals.
C -------------
C
C References:
C -----------
C R.Bacastow: "Numerical evaluation of the evasion factor", section 3.4
C of B.Bolin (ed): "Carbon cycle modelling" (SCOPE 16, 1981).
C
C Authors:
C --------
C E.Maier-Reimer, G.Hooss; MPI Me 8:1997.
C----------------------------------------------------------------

      IMPLICIT NONE
      REAL SIGC0, HS, C1, H0, HH, PCO2, CA, CS


      INTEGER NITER
      PARAMETER(NITER = 6)

#include "scparche.h"
#include "sparchem.h"

      REAL AINVA
      REAL ALPHAC, ALPHAK, ACQS, C0S
      REAL H1(NITER), AA(NITER), DADH(NITER), DELH(NITER)
      REAL CCC, DENOM(NITER)
      INTEGER ITER

C----------------------------------------------------------------------
C Version 6.1 autotune: compute ALPHAC & C0S from AOC, HS, MOL2GT, M2LIT
C here inside SIAM program instead of giving them as fixed parameters.
C AOC, MOL2GT, & M2LIT are new fixed parameters;
C HS & SIGC0 are new file input parameters.
C----------------------------------------------------------------------

      AINVA  = 1.0 / ALPHAA
      ALPHAC = AOC * HS * MOL2GT * M2LIT
      C0S    = ALPHAC * SIGC0
      ALPHAK = ALPHAS / ALPHAA + ALPHAC
      ACQS   = ALPHAC / ALPHAS

C----------------------------------------------------------------------
C Newton iteration for solution of the CO2 chemistry for given
C Alk and C1.
C----------------------------------------------------------------------

      CCC = C0S + W0 + C1
      H1(1) = H0

C----------------------------------------------------------------------
C Test output. Never do this during the SIAM optimization procedure.
c      WRITE(*,*)
c      WRITE(*,'(A2,3A16)') 'it','aa','h','del h'
C----------------------------------------------------------------------
C Tell TAMC that only final HH is needed.
C CADJ LOOP = ITERATION HH = MEMORY
C TAMC gives the following warning:
C "iteration solves non linear equation, NOT IMPLEMENTED YET"
C The best thing to do seems to compose at least a sequential loop:
C----------------------------------------------------------------------

CADJ LOOP = SEQUENTIAL

      DO ITER = 1, NITER-1

         DENOM(ITER) = (ALPHAK*H1(ITER)*H1(ITER)
     .           + ALPHAC*AK1*H1(ITER) + ALPHAC*AK1*AK2)

         AA(ITER)   = CCC * (AK1*H1(ITER)
     .          + 2*AK1*AK2) / DENOM(ITER)
     .          + AKB*BTS / (AKB+H1(ITER)) + AKW/H1(ITER)
     .          - H1(ITER) - ALK

         DADH(ITER) = CCC*AK1 / DENOM(ITER)
     .          - CCC * (AK1*H1(ITER) + 2*AK1*AK2) *
     .                  (2*ALPHAK*H1(ITER)
     .          + ALPHAC*AK1) / (DENOM(ITER) * DENOM(ITER))
     .          - AKB*BTS/((AKB+H1(ITER))*(AKB+H1(ITER)))
     .          - AKW/(H1(ITER)*H1(ITER)) - 1

         DELH(ITER) = AA(ITER) / DADH(ITER)

         H1(ITER+1) =  H1(ITER) - DELH(ITER)

C----------------------------------------------------------------------
C Test output. Avoid this during the SIAM optimization procedure.
c         WRITE(*,'(I2,3E16.6)') ITER, AA, H1(ITER), DELH
C----------------------------------------------------------------------

      END DO

      HH   = H1(NITER)

      PCO2 = CCC / (AINVA +
     .        ACQS * (1.0 + AK1/HH + AK1*AK2/(HH*HH)))

      CA = PCO2 * AINVA - W0
      CS = C1 - CA

      RETURN
      END

C######################################################################
